@@ -1,17 +1,6 @@
-#define __GNOME_FONT_FACE_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Copyright (C) 2000-2001 Ximian Inc. and authors
- *
- *  Authors:
- *    Jody Goldberg <jody@ximian.com>
- *    Miguel de Icaza <miguel@ximian.com>
- *    Lauris Kaplinski <lauris@ximian.com>
- *    Christopher James Lahey <clahey@ximian.com>
- *    Michael Meeks <michael@ximian.com>
- *    Morten Welinder <terra@diku.dk>
- *
- *  GnomeFontFace - unscaled typeface
+ *  gnome-font-face.c: unscaled typeface
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -26,10 +15,24 @@
  *  You should have received a copy of the GNU Library General Public
  *  License along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *  Authors:
+ *    Jody Goldberg <jody@ximian.com>
+ *    Miguel de Icaza <miguel@ximian.com>
+ *    Lauris Kaplinski <lauris@ximian.com>
+ *    Christopher James Lahey <clahey@ximian.com>
+ *    Michael Meeks <michael@ximian.com>
+ *    Morten Welinder <terra@diku.dk>
+ *
+ *  Copyright (C) 2000-2001 Ximian Inc. and authors
+ *
  */
 
+#define __GNOME_FONT_FACE_C__
+
 #include <config.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -38,10 +41,11 @@
 #include <fcntl.h>
 #include <stdarg.h>
 #include <locale.h>
-#include <libgnomeprint/gnome-print-i18n.h>
 #include <libgnomeprint/gnome-font-private.h>
+#include "gnome-print-i18n.h"
 #include "gp-fontmap.h"
 #include "gp-truetype-utils.h"
+#include "gp-tt-t1.h"
 #include "gnome-font-family.h"
 
 #include <freetype/ftoutln.h>
@@ -78,6 +82,10 @@ GFPSObject *gff_empty_pso (const GnomeFontFace *face, GPFontEntry *entry);
 static void gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...);
 static void gf_pso_ensure_space (GFPSObject * pso, gint size);
 
+#define GFE_IS_T1_A(e) (((e)->type == GP_FONT_ENTRY_TYPE1) || ((e)->type == GP_FONT_ENTRY_TYPE1_ALIAS))
+#define GFE_IS_TT(e) ((e)->type == GP_FONT_ENTRY_TRUETYPE)
+#define GFE_IS_S(e) ((e)->type == GP_FONT_ENTRY_SPECIAL)
+
 static GObjectClass * parent_class;
 
 GType
@@ -438,48 +446,104 @@ gnome_font_face_get_sample (GnomeFontFace * face)
 	return _("The quick brown fox jumps over the lazy dog.");
 }
 
+/*
+ * Creates new face and creates link with FontEntry
+ */
+
+static void
+gff_face_from_entry (GPFontEntry *e)
+{
+	GnomeFontFace *face;
+
+	g_return_if_fail (e->face == NULL);
+
+	face = g_object_new (GNOME_TYPE_FONT_FACE, NULL);
+
+	gp_font_entry_ref (e);
+	face->entry = e;
+	e->face = face;
+}
+
 GnomeFontFace *
 gnome_font_face_find (const guchar *name)
 {
 	GPFontMap * map;
 	GPFontEntry * e;
 
-	g_return_val_if_fail (name != NULL, NULL);
+	if (name) {
+		map = gp_fontmap_get ();
 
-	map = gp_fontmap_get ();
+		e = g_hash_table_lookup (map->fontdict, name);
+		if (!e) {
+			gp_fontmap_release (map);
+			return NULL;
+		}
+		if (e->face) {
+			gnome_font_face_ref (e->face);
+			gp_fontmap_release (map);
+			return e->face;
+		}
+
+		gff_face_from_entry (e);
 
-	e = g_hash_table_lookup (map->fontdict, name);
-	if (!e) {
 		gp_fontmap_release (map);
-		return NULL;
+
+		return (e->face);
+	} else {
+		return gnome_font_face_find_closest (NULL);
 	}
-	if (e->face) {
-		gnome_font_face_ref (e->face);
-		gp_fontmap_release (map);
-		return e->face;
-	}
-
-	e->face = g_object_new (GNOME_TYPE_FONT_FACE, NULL);
-	e->face->entry = e;
-	gp_font_entry_ref (e);
-
-	gp_fontmap_release (map);
-
-	return (e->face);
 }
 
+/* fixme: Do some well-known substitutions here */
+
 GnomeFontFace *
 gnome_font_face_find_closest (const guchar *name)
 {
 	GnomeFontFace *face;
+	GPFontMap *map;
 
-	if (name)
-		face = gnome_font_face_find (name);
-	else
-		face = NULL;
+	face = NULL;
 
-	if (!face)
-		face = gnome_font_face_find_closest_from_weight_slant ("Helvetica", GNOME_FONT_BOOK, FALSE);
+	map = gp_fontmap_get ();
+
+	if (name) face = gnome_font_face_find (name);
+
+	if (!face) {
+		/* No match, try default */
+		guchar *locale, *p;
+		guchar c[128];
+		GPFontEntry *e;
+		locale = setlocale (LC_ALL, NULL);
+		if (!locale) locale = "C";
+		strncpy (c, locale, 127);
+		c[127] = '\0';
+		p = c;
+		while (isalpha (*p) || (*p == '_')) p += 1;
+		if (*p) *p = '\0';
+		e = g_hash_table_lookup (map->defaultsdict, c);
+		if (!e) e = g_hash_table_lookup (map->defaultsdict, "C");
+		if (e) {
+			if (e->face) {
+				gnome_font_face_ref (e->face);
+			} else {
+				gff_face_from_entry (e);
+			}
+			face = e->face;
+		}
+	}
+	if (!face && map->fonts) {
+		/* No face, no default, load whatever font is first */
+		GPFontEntry *e;
+		e = (GPFontEntry *) map->fonts->data;
+		if (e->face) {
+			gnome_font_face_ref (e->face);
+		} else {
+			gff_face_from_entry (e);
+		}
+		face = e->face;
+	}
+
+	gp_fontmap_release (map);
 
 	return face;
 }
@@ -500,11 +564,6 @@ gnome_font_face_find_closest_from_weight_slant (const guchar *family, GnomeFontW
 
 	g_return_val_if_fail (family != NULL, NULL);
 
-#if 0
-	if (!fontmap) gff_refresh_fontmap ();
-	g_return_val_if_fail (fontmap != NULL, NULL);
-#endif
-
 	/* This should be reimplemented to use the gnome_font_family_hash. */
 
 	map = gp_fontmap_get ();
@@ -517,41 +576,21 @@ gnome_font_face_find_closest_from_weight_slant (const guchar *family, GnomeFontW
 		entry = (GPFontEntry *) l->data;
 		if (!strcasecmp (family, entry->familyname)) {
 			if (entry->type == GP_FONT_ENTRY_ALIAS) entry = ((GPFontEntryAlias *) entry)->ref;
-			if (entry->type == GP_FONT_ENTRY_TYPE1) {
-				GPFontEntryT1 * t1;
-				t1 = (GPFontEntryT1 *) entry;
-				dist = abs (weight - t1->Weight) +
-					100 * (italic != (t1->ItalicAngle != 0));
-				if (dist < best_dist) {
-					best_dist = dist;
-					best = entry;
-				}
-			} else if (entry->type == GP_FONT_ENTRY_TRUETYPE) {
-				GPFontEntryTT * tt;
-				tt = (GPFontEntryTT *) entry;
-				dist = abs (weight - tt->Weight) +
-					100 * (italic != (tt->ItalicAngle != 0));
-				if (dist < best_dist) {
-					best_dist = dist;
-					best = entry;
-				}
+			dist = abs (weight - entry->Weight) +
+				100 * (italic != (entry->ItalicAngle != 0));
+			if (dist < best_dist) {
+				best_dist = dist;
+				best = entry;
 			}
 		}
 	}
 
-	if (best) {
-		face = gnome_font_face_find (best->name);
-	} else {
-		face = gnome_font_face_find ("Helvetica");
-	}
-
-	if (!face && map->fonts) {
-		face = gnome_font_face_find (((GPFontEntry *) map->fonts->data)->name);
-	}
+	if (best) face = gnome_font_face_find (best->name);
 
 	gp_fontmap_release (map);
 
-	g_return_val_if_fail (face != NULL, NULL);
+	/* If nothing found, go with default */
+	if (!face) return gnome_font_face_find (NULL);
 
 	return face;
 }
@@ -567,10 +606,10 @@ gnome_font_face_find_from_family_and_style (const guchar *family, const guchar *
 	g_return_val_if_fail (style != NULL, NULL);
 
 	gff = gnome_font_family_new (family);
-	if (!gff) return gnome_font_face_find_closest_from_weight_slant ("Helvetica", GNOME_FONT_BOOK, FALSE);
+	if (!gff) return gnome_font_face_find (NULL);
 	face = gnome_font_family_get_face_by_stylename (gff, style);
 	gnome_font_family_unref (gff);
-	if (!face) return gnome_font_face_find_closest_from_weight_slant ("Helvetica", GNOME_FONT_BOOK, FALSE);
+	if (!face) return gnome_font_face_find (NULL);
 
 	return face;
 }
@@ -637,12 +676,12 @@ gnome_font_style_list_free (GList *styles)
 }
 
 GnomeFont *
-gnome_font_face_get_font (GnomeFontFace * face, gdouble size, gdouble xres, gdouble yres)
+gnome_font_face_get_font (GnomeFontFace *face, gdouble size, gdouble xres, gdouble yres)
 {
 	g_return_val_if_fail (face != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
 
-	return gnome_font_find (face->entry->name, size);
+	return gnome_font_face_get_font_full (face, size, NULL);
 }
 
 GnomeFont *
@@ -715,7 +754,7 @@ gff_load (GnomeFontFace *face)
 		entry = face->entry;
 	}
 
-	if (entry->type == GP_FONT_ENTRY_TYPE1) {
+	if (GFE_IS_T1_A (entry)) {
 		GPFontEntryT1 *t1;
 		t1 = (GPFontEntryT1 *) entry;
 		ft_result = FT_New_Face (ft_library, t1->pfb.name, 0, &ft_face);
@@ -726,11 +765,24 @@ gff_load (GnomeFontFace *face)
 				g_warning ("file %s: line %d: face %s: Cannot attach file %s", __FILE__, __LINE__, face->entry->name, t1->afm.name);
 			}
 		}
-	} else if (entry->type == GP_FONT_ENTRY_TRUETYPE) {
+	} else if (GFE_IS_TT (entry)) {
 		GPFontEntryTT *tt;
 		tt = (GPFontEntryTT *) entry;
-		ft_result = FT_New_Face (ft_library, tt->ttf.name, 0, &ft_face);
+		ft_result = FT_New_Face (ft_library, tt->ttf.name, tt->facenum, &ft_face);
 		g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
+	} else if (GFE_IS_S (entry)) {
+		GPFontEntrySpecial *s;
+		GSList *l;
+		s = (GPFontEntrySpecial *) entry;
+		ft_result = FT_New_Face (ft_library, s->file.name, 0, &ft_face);
+		g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
+		for (l = s->additional; l != NULL; l = l->next) {
+			ft_result = FT_Attach_File (ft_face, (gchar *) l->data);
+			if (ft_result != FT_Err_Ok) {
+				g_warning ("file %s: line %d: face %s: Cannot attach file %s",
+					   __FILE__, __LINE__, face->entry->name, (gchar *) l->data);
+			}
+		}
 	} else {
 		g_assert_not_reached ();
 	}
@@ -740,7 +792,10 @@ gff_load (GnomeFontFace *face)
 
 	face->ft_face = ft_face;
 
-	FT_Select_Charmap (ft_face, ft_encoding_unicode);
+	ft_result = FT_Select_Charmap (ft_face, ft_encoding_unicode);
+	if (ft_result != FT_Err_Ok) {
+		g_warning ("file %s: line %d: Face %s does not have unicode charmap", __FILE__, __LINE__, face->entry->name);
+	}
 
 	face->num_glyphs = ft_face->num_glyphs;
 	g_return_val_if_fail (face->num_glyphs > 0, FALSE);
@@ -807,17 +862,16 @@ typedef struct {
 static int gfft2_move_to (FT_Vector * to, void * user)
 {
 	GFFT2OutlineData * od;
-	ArtBpath * s;
 	ArtPoint p;
 
 	od = (GFFT2OutlineData *) user;
 
-	s = &od->bp[od->end - 1];
-
 	p.x = to->x * od->t[0] + to->y * od->t[2];
 	p.y = to->x * od->t[1] + to->y * od->t[3];
 
-	if ((p.x != s->x3) || (p.y != s->y3)) {
+	if (od->end == 0 ||
+	    p.x != od->bp[od->end - 1].x3 ||
+	    p.y != od->bp[od->end - 1].y3) {
 		od->bp[od->end].code = ART_MOVETO;
 		od->bp[od->end].x3 = to->x * od->t[0] + to->y * od->t[2];
 		od->bp[od->end].y3 = to->x * od->t[1] + to->y * od->t[3];
@@ -926,342 +980,144 @@ gff_ol2bp (FT_Outline * ol, gdouble transform[])
 	return od.bp;
 }
 
-#if 0
-#define KERN_PAIR_HASH(g1, g2) ((g1) * 367 + (g2) * 31)
+/* PSO (PostScriptObject stuff */
 
-static gboolean
-gff_load_afm (GnomeFontFace * face)
-{
-	GnomeFontFacePrivate * priv;
-	GPFontEntryT1 * t1;
-	Font_Info *fi;
-	FILE *afm_f;
-	int status;
-	gint nglyphs;
-	gint privcode;
-
-	g_return_val_if_fail ((face->private->entry->type == GP_FONT_ENTRY_TYPE1) ||
-			      (face->private->entry->type == GP_FONT_ENTRY_TYPE1_ALIAS),
-			      FALSE);
-
-	priv = face->private;
-	t1 = (GPFontEntryT1 *) face->private->entry;
-
-	afm_f = fopen (t1->afm.name, "r");
-	if (afm_f != NULL) {
-		status = parseFile (afm_f, &fi, P_G | P_M | P_P);
-#ifdef VERBOSE
-		debugmsg (_("status loading %s = %d\n"), t1->afm.name, status);
-#endif
-		if (status == 0) {
-			GnomeFontLigList **ligtab;
-			Ligature *ligs;
-			int i;
-			GnomeFontKernPair *ktab;
-			int ktabsize;
-
-			priv->ascender = fi->gfi->ascender;
-			/* This is negated because adobe fonts use a negative number
-			   for descenders below the baseline, but most people expect
-			   a positive number for descenders below the line. */
-			priv->descender = -fi->gfi->descender;
-			priv->fixed_width = fi->gfi->isFixedPitch;
-
-			/* Added for pdf */
-			priv->capheight = (gdouble) fi->gfi->capHeight;
-			priv->italics_angle = (gdouble) fi->gfi->italicAngle;
-			priv->xheight = (gdouble) fi->gfi->xHeight;
-			priv->bbox.x0 = (gdouble) fi->gfi->fontBBox.llx;
-			priv->bbox.y0 = (gdouble) fi->gfi->fontBBox.lly;
-			priv->bbox.x1 = (gdouble) fi->gfi->fontBBox.urx;
-			priv->bbox.y1 = (gdouble) fi->gfi->fontBBox.ury;
-			/* */
-	  
-			priv->underline_position = fi->gfi->underlinePosition;
-			priv->underline_thickness = fi->gfi->underlineThickness;
-
-			ligtab = g_malloc (256 * sizeof (GnomeFontLigList *));
-			priv->ligs = ligtab;
-
-			for (i = 0; i < 256; i++) {
-				ligtab[i] = 0;
-			}
-
-			priv->glyphs = g_new0 (GFFGlyphInfo, fi->numOfChars + 1);
-			priv->unimap = gp_uc_map_new ();
-#if 0
-			priv->glyphmap = g_hash_table_new (NULL, NULL);
-#endif
-
-			gff_fill_zero_glyph (face);
-
-			nglyphs = 1;
-			privcode = 0xe000;
-
-			for (i = 0; i < fi->numOfChars; i++) {
-				CharMetricInfo * info;
-				gint unicode;
-				gchar * constname;
-
-				constname = NULL; /* Kill warning */
-				info = fi->cmi + i;
-
-				unicode = gp_unicode_from_ps (info->name);
-				if (unicode < 1) unicode = gp_unicode_from_dingbats (info->name);
-
-				if (unicode < 1) {
-					if (!priv->privencoding) priv->privencoding = g_hash_table_new (g_str_hash, g_str_equal);
-					if (!g_hash_table_lookup (priv->privencoding, info->name)) {
-						unicode = privcode++;
-						constname = g_strdup (info->name);
-						g_hash_table_insert (priv->privencoding, constname, GINT_TO_POINTER (unicode));
-					}
-				} else {
-					constname = (gchar *) gp_const_ps_from_ps (info->name);
-				}
-
-				if (unicode) {
-					GFFGlyphInfo * gi;
-
-					gi = priv->glyphs + nglyphs;
-
-					gi->unicode = unicode;
-					gi->psname = constname;
-
-					gi->advance.x = info->wx;
-					gi->advance.y = info->wy;
-
-					gi->bbox.x0 = info->charBBox.llx;
-					gi->bbox.y0 = info->charBBox.lly;
-					gi->bbox.x1 = info->charBBox.urx;
-					gi->bbox.y1 = info->charBBox.ury;
-
-					if (gp_multi_from_ps (info->name)) {
-						const GSList * l;
-						l = gp_multi_from_ps (info->name);
-						while (l) {
-							gp_uc_map_insert (priv->unimap, GPOINTER_TO_INT (l->data), nglyphs);
-#if 0
-							gff_assign_glyph (face, GPOINTER_TO_INT (l->data), nglyphs);
-#endif
-							l = l->next;
-						}
-					} else {
-						gp_uc_map_insert (priv->unimap, unicode, nglyphs);
-#if 0
-						gff_assign_glyph (face, unicode, nglyphs);
-#endif
-					}
-
-					nglyphs++;
-				}
-			}
-
-			priv->glyphs = g_renew (GFFGlyphInfo, priv->glyphs, nglyphs);
-			priv->num_glyphs = nglyphs;
-
-	  for (i = 0; i < fi->numOfChars; i++)
-	    {
-	      int code;
-
-	      /* Get the width */
-	      code = fi->cmi[i].code;
-	      if (code >= 0 && code < 256)
-		{
-		  /* Get the ligature info */
-		  for (ligs = fi->cmi[i].ligs; ligs != NULL; ligs = ligs->next)
-		    {
-		      int succ, lig;
-		      GnomeFontLigList *ll;
-		      gint u;
-
-		      /* We assume here that dingbats do not have ligatures */
-		      u = gp_unicode_from_ps (ligs->succ);
-		      succ = gnome_font_face_lookup_default (face, u);
-#if 0
-		      succ = GPOINTER_TO_INT (g_hash_table_lookup (face->private->glyphmap, GINT_TO_POINTER (u)));
-#endif
-		      u = gp_unicode_from_ps (ligs->succ);
-		      lig = gnome_font_face_lookup_default (face, u);
-#if 0
-		      lig = GPOINTER_TO_INT (g_hash_table_lookup (face->private->glyphmap, GINT_TO_POINTER (u)));
-#endif
-		      if ((succ > 0) && (lig > 0))
-			{
-			  ll = g_new (GnomeFontLigList, 1);
-			  ll->succ = succ;
-			  ll->lig = lig;
-			  ll->next = ligtab[code];
-			  ligtab[code] = ll;
-			}
-		    }
-		}
-	    }
-
-	  /* process the kern pairs */
-	  for (ktabsize = 1; ktabsize < fi->numOfPairs << 1; ktabsize <<= 1);
-	  ktab = g_new (GnomeFontKernPair, ktabsize);
-	  face->private->kerns = ktab;
-	  face->private->num_kerns = ktabsize;
-	  for (i = 0; i < ktabsize; i++)
-	    {
-	      ktab[i].glyph1 = -1;
-	      ktab[i].glyph2 = -1;
-	      ktab[i].x_amt = 0;
-	    }
-
-	  for (i = 0; i < fi->numOfPairs; i++)
-	    {
-	      int glyph1, glyph2;
-	      int j;
-	      gint u;
-
-	      /* We assume here that dingbats do not have kerning */
-	      u = gp_unicode_from_ps (fi->pkd[i].name1);
-	      glyph1 = gnome_font_face_lookup_default (face, u);
-#if 0
-	      glyph1 = GPOINTER_TO_INT (g_hash_table_lookup (face->private->glyphmap, GINT_TO_POINTER (u)));
-#endif
-	      u = gp_unicode_from_ps (fi->pkd[i].name2);
-	      glyph2 = gnome_font_face_lookup_default (face, u);
-#if 0
-	      glyph2 = GPOINTER_TO_INT (g_hash_table_lookup (face->private->glyphmap, GINT_TO_POINTER (u)));
-#endif
-
-	      for (j = KERN_PAIR_HASH (glyph1, glyph2) & (ktabsize - 1);
-		   ktab[j].glyph1 != -1;
-		   j = (j + 1) & (ktabsize - 1));
-	      ktab[j].glyph1 = glyph1;
-	      ktab[j].glyph2 = glyph2;
-	      ktab[j].x_amt = fi->pkd[i].xamt;
-#ifdef VERBOSE
-	      debugmsg ("kern pair %s(%d) %s(%d) %d\n",
-		       fi->pkd[i].name1, glyph1,
-		       fi->pkd[i].name2, glyph2,
-		       fi->pkd[i].xamt);
-#endif
-	    }
-
-#ifdef VERBOSE
-	  for (i = 0; i < ktabsize; i++)
-	    debugmsg ("%d %d %d\n",
-		     ktab[i].glyph1, ktab[i].glyph2, ktab[i].x_amt);
-#endif
-	}
-
-      if (fi)
-	parseFileFree (fi);
-
-      fclose (afm_f);
-
-      return TRUE;
-    }
-
-  return FALSE;
-}
-
-static void
-gff_fill_zero_glyph (GnomeFontFace * face)
-{
-	GFFGlyphInfo * info;
-	static ArtBpath undef[] = {{ART_MOVETO, 0.0, 0.0, 0.0, 0.0, 50.0, 50.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 750.0, 50.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 750.0, 950.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 50.0, 950.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 50.0, 50.0},
-				   {ART_MOVETO, 0.0, 0.0, 0.0, 0.0, 100.0, 100.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 700.0, 100.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 700.0, 900.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 100.0, 900.0},
-				   {ART_LINETO, 0.0, 0.0, 0.0, 0.0, 100.0, 100.0},
-				   {ART_END, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};
-
-	info = face->private->glyphs;
-
-	info->unicode = 0;
-	info->psname = ".notdef";
-
-	info->advance.x = 800.0;
-	info->advance.y = 0.0;
-
-	info->bbox.x0 = 50.0;
-	info->bbox.y0 = 50.0;
-	info->bbox.x1 = 750.0;
-	info->bbox.y1 = 900.0;
-
-	info->bpath = undef;
-}
-#endif
+static void gff_pso_ensure_buffer_t1 (GFPSObject *pso);
+static void gff_pso_ensure_buffer_tt (GFPSObject *pso);
+static void gff_pso_ensure_buffer_empty (GFPSObject *pso);
+static void gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...);
+static void gf_pso_ensure_space (GFPSObject *pso, gint size);
 
 /*
- * Create downloadable PS representation of given face
+ * Generates intial PSObject
  */
 
 GFPSObject *
-gnome_font_face_create_ps_object (const GnomeFontFace * face)
+gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname)
 {
-	GPFontEntry *entry;
+	GFPSObject *pso;
 
 	g_return_val_if_fail (face != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
 
-	if (face->entry->type == GP_FONT_ENTRY_ALIAS) {
-		entry = ((GPFontEntryAlias *) face->entry)->ref;
+	pso = g_new0 (GFPSObject, 1);
+
+	pso->face = face;
+	g_object_ref (G_OBJECT (face));
+	if (residentname) pso->residentname = g_strdup (residentname);
+	pso->encodedname = g_strdup_printf ("GnomeUni-%s", face->entry->psname);
+	pso->bufsize = 0;
+	pso->length = 0;
+	pso->buf = NULL;
+	if (GFF_LOADED (face)) {
+		pso->encodedbytes = (face->num_glyphs < 256) ? 1 : 2;
+		pso->num_glyphs = face->num_glyphs;
+		pso->glyphs = g_new0 (guint32, (pso->num_glyphs + 32) >> 5);
 	} else {
-		entry = face->entry;
+		g_warning ("file %s: line %d: Face: %s: Cannot load face", __FILE__, __LINE__, face->entry->name);
+		pso->encodedbytes = 1;
+		pso->num_glyphs = 1;
+		pso->glyphs = NULL;
+		gff_pso_ensure_buffer_empty (pso);
 	}
 
-	if (!GFF_LOADED (face)) {
-		g_warning ("file %s: line %d: Face %s: Cannot load face", __FILE__, __LINE__, face->entry->name);
-		/* fixme: create empty font */
-		return gff_empty_pso (face, entry);
-	}
+	return pso;
+}
 
-	switch (entry->type) {
+#define PSO_GLYPH_MARKED(pso,g) (pso->glyphs[(g) >> 5] & (1 << ((g) & 0x1f)))
+
+void
+gnome_font_face_pso_mark_glyph (GFPSObject *pso, gint glyph)
+{
+	g_return_if_fail (pso != NULL);
+
+	if (!pso->glyphs) return;
+
+	glyph = CLAMP (glyph, 0, pso->num_glyphs);
+
+	pso->glyphs[glyph >> 5] |= (1 << (glyph & 0x1f));
+}
+
+void
+gnome_font_face_pso_free (GFPSObject *pso)
+{
+	g_return_if_fail (pso != NULL);
+
+	if (pso->face) g_object_unref (G_OBJECT (pso->face));
+	if (pso->residentname) g_free (pso->residentname);
+	if (pso->encodedname) g_free (pso->encodedname);
+	if (pso->glyphs) g_free (pso->glyphs);
+	if (pso->buf) g_free (pso->buf);
+	g_free (pso);
+}
+
+void
+gnome_font_face_pso_ensure_buffer (GFPSObject *pso)
+{
+	g_return_if_fail (pso != NULL);
+
+	switch (pso->face->entry->type) {
 	case GP_FONT_ENTRY_TYPE1:
-		return gff_pso_from_type1 (face, entry);
+	case GP_FONT_ENTRY_TYPE1_ALIAS:
+		gff_pso_ensure_buffer_t1 (pso);
 		break;
 	case GP_FONT_ENTRY_TRUETYPE:
-		return gff_pso_from_truetype (face, entry);
+		gff_pso_ensure_buffer_tt (pso);
 		break;
 	default:
-		g_assert_not_reached ();
+		g_warning ("file %s: line %d: Unknown face entry type %s:%d",
+			   __FILE__, __LINE__, pso->face->entry->name, pso->face->entry->type);
+		gff_pso_ensure_buffer_empty (pso);
 		break;
 	}
-
-	return gff_empty_pso (face, entry);
 }
 
 #define INT32_LSB(q) ((q)[0] + ((q)[1] << 8) + ((q)[2] << 16) + ((q)[3] << 24))
 
-GFPSObject *
-gff_pso_from_type1 (const GnomeFontFace *face, GPFontEntry *entry)
+static void
+gff_pso_ensure_buffer_t1 (GFPSObject *pso)
 {
-	GPFontEntryT1 * t1;
-	GFPSObject * pso;
+	GPFontEntryT1 *t1;
 	struct stat st;
 	guchar * fbuf;
 	gint fh;
-	const gchar * embeddedname;
+	const gchar *embeddedname;
 
-	t1 = (GPFontEntryT1 *) entry;
+	t1 = (GPFontEntryT1 *) pso->face->entry;
 
-	/* mmap file */
-	if (stat (t1->pfb.name, &st) != 0) return gff_empty_pso (face, entry);
+	if (pso->face->entry->type == GP_FONT_ENTRY_TYPE1) {
+		embeddedname = pso->face->entry->psname;
+	} else {
+		embeddedname = ((GPFontEntryT1Alias *) pso->face->entry)->alias;
+	}
+
+	if (!GFF_LOADED (pso->face)) {
+		g_warning ("file %s: line %d: Face: %s: Cannot load face", __FILE__, __LINE__, pso->face->entry->name);
+		gff_pso_ensure_buffer_empty (pso);
+		return;
+	}
+
+	/* Step 0: mmap pfb file */
 	fh = open (t1->pfb.name, O_RDONLY);
-	if (fh < 0) return gff_empty_pso (face, entry);
+	if (fh < 0) {
+		g_warning ("file %s: line %d: Cannot open font file %s", __FILE__, __LINE__, t1->pfb.name);
+		gff_pso_ensure_buffer_empty (pso);
+		return;
+	}
+	if (fstat (fh, &st) != 0) {
+		g_warning ("file %s: line %d: Cannot fstat font file %s", __FILE__, __LINE__, t1->pfb.name);
+		gff_pso_ensure_buffer_empty (pso);
+		return;
+	}
 	fbuf = mmap (NULL, st.st_size, PROT_READ, MAP_SHARED, fh, 0);
 	close (fh);
-	if (fbuf == NULL) return gff_empty_pso (face, entry);
-	if (fbuf == (guchar *) -1) return gff_empty_pso (face, entry);
-
-	pso = g_new (GFPSObject, 1);
-	pso->bufsize = st.st_size + 1024;
-	pso->length = 0;
-	pso->buf = g_new (guchar, pso->bufsize);
-	pso->encodedname = NULL;
-	pso->encodedbytes = 0;
+	if ((fbuf == NULL) || (fbuf == (guchar *) -1)) {
+		g_warning ("file %s: line %d: Cannot open font file %s", __FILE__, __LINE__, t1->pfb.name);
+		gff_pso_ensure_buffer_empty (pso);
+		return;
+	}
 
+	/* Step 1: */	   
 	if (*fbuf == 0x80) {
 		const char hextab[16] = "0123456789abcdef";
 		gint idx;
@@ -1273,11 +1129,9 @@ gff_pso_from_type1 (const GnomeFontFace *face, GPFontEntry *entry)
 		while (idx < st.st_size) {
 			gint length, i;
 			if (fbuf[idx] != 0x80) {
-				g_warning ("Corrupted pfb file %s", t1->pfb.name);
-				munmap (fbuf, st.st_size);
-				g_free (pso->buf);
-				g_free (pso);
-				return gff_empty_pso (face, entry);
+				g_warning ("file %s: line %d: Corrupt %s", __FILE__, __LINE__, t1->pfb.name);
+				gff_pso_ensure_buffer_empty (pso);
+				return;
 			}
 			switch (fbuf[idx + 1]) {
 			case 1:
@@ -1306,29 +1160,24 @@ gff_pso_from_type1 (const GnomeFontFace *face, GPFontEntry *entry)
 				idx = st.st_size;
 				break;
 			default:
-				g_warning ("Corrupted pfb file %s", t1->pfb.name);
-				munmap (fbuf, st.st_size);
-				g_free (pso->buf);
-				g_free (pso);
-				return gff_empty_pso (face, entry);
-				break;
+				g_warning ("file %s: line %d: Corrupt %s", __FILE__, __LINE__, t1->pfb.name);
+				gff_pso_ensure_buffer_empty (pso);
+				return;
 			}
 		}
 	} else {
+		gf_pso_ensure_space (pso, st.st_size);
 		memcpy (pso->buf, fbuf, st.st_size);
 		pso->buf[st.st_size] = '\0';
 		pso->length = st.st_size;
 	}
 
+	/* Unmap file */
 	munmap (fbuf, st.st_size);
 
-	embeddedname = entry->psname;
-	pso->encodedname = g_strdup_printf ("GnomeUni-%s", embeddedname);
-
-	if (face->num_glyphs < 256) {
+	if (pso->encodedbytes == 1) {
 		gint glyph;
 		/* 8-bit vector */
-		pso->encodedbytes = 1;
 		gf_pso_sprintf (pso, "/%s findfont dup length dict begin\n", embeddedname);
 		gf_pso_sprintf (pso, "{1 index /FID ne {def} {pop pop} ifelse} forall\n");
 		gf_pso_sprintf (pso, "/Encoding [\n");
@@ -1336,13 +1185,11 @@ gff_pso_from_type1 (const GnomeFontFace *face, GPFontEntry *entry)
 			guint g;
 			gchar c[256];
 			FT_Error status;
-			g = (glyph < face->num_glyphs) ? glyph : 0;
-			status = FT_Get_Glyph_Name (face->ft_face, glyph, c, 256);
+			g = (glyph < pso->face->num_glyphs) ? glyph : 0;
+			status = FT_Get_Glyph_Name (pso->face->ft_face, g, c, 256);
 			if (status != FT_Err_Ok) {
-				g_warning ("Cannot get name of glyph %d", glyph);
-				g_free (pso->buf);
-				g_free (pso);
-				return gff_empty_pso (face, entry);
+				g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, g, t1->pfb.name);
+				g_snprintf (c, 256, ".notdef");
 			}
 			gf_pso_sprintf (pso, ((glyph & 0xf) == 0xf) ? "/%s\n" : "/%s ", c);
 		}
@@ -1351,8 +1198,7 @@ gff_pso_from_type1 (const GnomeFontFace *face, GPFontEntry *entry)
 	} else {
 		gint nfonts, nglyphs, i, j;
 		/* 16-bit vector */
-		pso->encodedbytes = 2;
-		nglyphs = face->num_glyphs;
+		nglyphs = pso->face->num_glyphs;
 		nfonts = (nglyphs + 255) >> 8;
 
 		gf_pso_sprintf (pso, "32 dict begin\n");
@@ -1377,12 +1223,10 @@ gff_pso_from_type1 (const GnomeFontFace *face, GPFontEntry *entry)
 				FT_Error status;
 				glyph = 256 * i + j;
 				if (glyph >= nglyphs) glyph = 0;
-				status = FT_Get_Glyph_Name (face->ft_face, glyph, c, 256);
+				status = FT_Get_Glyph_Name (pso->face->ft_face, glyph, c, 256);
 				if (status != FT_Err_Ok) {
-					g_warning ("Cannot get name of glyph %d", glyph);
-					g_free (pso->buf);
-					g_free (pso);
-					return gff_empty_pso (face, entry);
+					g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, glyph, t1->pfb.name);
+					g_snprintf (c, 256, ".notdef");
 				}
 				gf_pso_sprintf (pso, ((j & 0xf) == 0xf) ? "/%s\n" : "/%s ", c);
 			}
@@ -1400,8 +1244,6 @@ gff_pso_from_type1 (const GnomeFontFace *face, GPFontEntry *entry)
 		gf_pso_sprintf (pso, "currentdict end\n");
 		gf_pso_sprintf (pso, "/%s exch definefont pop\n", pso->encodedname);
 	}
-
-	return pso;
 }
 
 /*
@@ -1410,13 +1252,12 @@ gff_pso_from_type1 (const GnomeFontFace *face, GPFontEntry *entry)
  * Adobe Inc., The Type42 Font Format Specification, 1998 <http://partners.adobe.com/asn/developer/PDFS/TN/5012.Type42_Spec.pdf>
  */
 
-GFPSObject *
-gff_pso_from_truetype (const GnomeFontFace *face, GPFontEntry *entry)
+static void
+gff_pso_ensure_buffer_tt (GFPSObject *pso)
 {
-	GPFontEntryTT * tt;
-	GFPSObject * pso;
+	GPFontEntryTT *tt;
 	struct stat st;
-	guchar * fbuf;
+	guchar *fbuf;
 	gint fh;
 	GSList *strings;
 	const gchar * embeddedname;
@@ -1424,77 +1265,92 @@ gff_pso_from_truetype (const GnomeFontFace *face, GPFontEntry *entry)
 	const ArtDRect *bbox;
 	gint i;
 
-	tt = (GPFontEntryTT *) entry;
+	tt = (GPFontEntryTT *) pso->face->entry;
 
 	/* mmap file */
-	if (stat (tt->ttf.name, &st) != 0) return gff_empty_pso (face, entry);
 	fh = open (tt->ttf.name, O_RDONLY);
-	if (fh < 0) return gff_empty_pso (face, entry);
+	if (fh < 0) {
+		g_warning ("file %s: line %d: Cannot open font file %s", __FILE__, __LINE__, tt->ttf.name);
+		gff_pso_ensure_buffer_empty (pso);
+		return;
+	}
+	if (fstat (fh, &st) != 0) {
+		g_warning ("file %s: line %d: Face: %s: Cannot fstat face", __FILE__, __LINE__, tt->ttf.name);
+		gff_pso_ensure_buffer_empty (pso);
+		return;
+	}
 	fbuf = mmap (NULL, st.st_size, PROT_READ, MAP_SHARED, fh, 0);
 	close (fh);
-	if (fbuf == NULL) return gff_empty_pso (face, entry);
-	if (fbuf == (guchar *) -1) return gff_empty_pso (face, entry);
+	if ((fbuf == NULL) || (fbuf == (guchar *) -1)) {
+		g_warning ("file %s: line %d: Cannot open font file %s", __FILE__, __LINE__, tt->ttf.name);
+		gff_pso_ensure_buffer_empty (pso);
+		return;
+	}
+
+	embeddedname = pso->face->entry->psname;
 
 	strings = gp_tt_split_file (fbuf, st.st_size);
-	if (strings == NULL) {
-		munmap (fbuf, st.st_size);
-		return gff_empty_pso (face, entry);
-	}
-
-	/* Create PSO */
-	pso = g_new (GFPSObject, 1);
-	pso->bufsize = st.st_size + 1024;
-	pso->length = 0;
-	pso->buf = g_new (guchar, pso->bufsize);
-	pso->encodedname = g_strdup_printf ("GnomeUni-%s", entry->psname);
-	pso->encodedbytes = 0;
-
-	embeddedname = entry->psname;
-
-	/* Download master font */
-	TTVersion = 1.0;
-	MfrRevision = 1.0;
-	gf_pso_sprintf (pso, "%%!PS-TrueTypeFont-%g-%g\n", TTVersion, MfrRevision);
-	gf_pso_sprintf (pso, "11 dict begin\n");
-	gf_pso_sprintf (pso, "/FontName /%s def\n", embeddedname);
-	gf_pso_sprintf (pso, "/Encoding 256 array\n");
-	gf_pso_sprintf (pso, "0 1 255 {1 index exch /.notdef put} for\n");
-	gf_pso_sprintf (pso, "readonly def\n");
-	gf_pso_sprintf (pso, "/PaintType 0 def\n");
-	gf_pso_sprintf (pso, "/FontMatrix [1 0 0 1 0 0] def\n");
-	/* fixme: */
-	bbox = gnome_font_face_get_stdbbox (face);
-	gf_pso_sprintf (pso, "/FontBBox [%g %g %g %g] def\n", bbox->x0, bbox->y0, bbox->x1, bbox->y1);
-	gf_pso_sprintf (pso, "/FontType 42 def\n");
-	/* fixme: XUID */
-	/* fixme: Be more intelligent */
-	gf_pso_sprintf (pso, "/sfnts [\n");
-	while (strings) {
-		guint start, next, size, i;
-		start = GPOINTER_TO_UINT (strings->data);
-		strings = g_slist_remove (strings, strings->data);
-		next = strings ? GPOINTER_TO_UINT (strings->data) : st.st_size;
-		size = next - start;
-		gf_pso_sprintf (pso, "<\n");
-		for (i = start; i < next; i+= 32) {
-			gint e, j;
-			e = MIN (i + 32, next);
-			for (j = i; j < e; j++) {
-				gf_pso_sprintf (pso, "%.2x", *(fbuf + j));
+	if (strings) {
+		/* Relatively easy - just embed TTF into PS stream */
+		/* Download master font */
+		TTVersion = 1.0;
+		MfrRevision = 1.0;
+		gf_pso_sprintf (pso, "%%!PS-TrueTypeFont-%g-%g\n", TTVersion, MfrRevision);
+		gf_pso_sprintf (pso, "11 dict begin\n");
+		gf_pso_sprintf (pso, "/FontName /%s def\n", embeddedname);
+		gf_pso_sprintf (pso, "/Encoding 256 array\n");
+		gf_pso_sprintf (pso, "0 1 255 {1 index exch /.notdef put} for\n");
+		gf_pso_sprintf (pso, "readonly def\n");
+		gf_pso_sprintf (pso, "/PaintType 0 def\n");
+		gf_pso_sprintf (pso, "/FontMatrix [1 0 0 1 0 0] def\n");
+		/* fixme: */
+		bbox = gnome_font_face_get_stdbbox (pso->face);
+		gf_pso_sprintf (pso, "/FontBBox [%g %g %g %g] def\n", bbox->x0, bbox->y0, bbox->x1, bbox->y1);
+		gf_pso_sprintf (pso, "/FontType 42 def\n");
+		/* fixme: XUID */
+		/* fixme: Be more intelligent */
+		gf_pso_sprintf (pso, "/sfnts [\n");
+		while (strings) {
+			guint start, next, size, i;
+			start = GPOINTER_TO_UINT (strings->data);
+			strings = g_slist_remove (strings, strings->data);
+			next = strings ? GPOINTER_TO_UINT (strings->data) : st.st_size;
+			size = next - start;
+			gf_pso_sprintf (pso, "<\n");
+			for (i = start; i < next; i+= 32) {
+				gint e, j;
+				e = MIN (i + 32, next);
+				for (j = i; j < e; j++) {
+					gf_pso_sprintf (pso, "%.2x", *(fbuf + j));
+				}
+				gf_pso_sprintf (pso, "\n");
 			}
-			gf_pso_sprintf (pso, "\n");
+			gf_pso_sprintf (pso, strings ? ">\n" : "00>\n");
 		}
-		gf_pso_sprintf (pso, strings ? ">\n" : "00>\n");
+		gf_pso_sprintf (pso, "] def\n");
+		/* fixme: Use CID or something */
+		gf_pso_sprintf (pso, "/CharStrings %d dict dup begin\n", pso->face->num_glyphs);
+		gf_pso_sprintf (pso, "/.notdef 0 def\n");
+		for (i = 1; i < pso->face->num_glyphs; i++) {
+			gf_pso_sprintf (pso, "/_%d %d def\n", i, i);
+		}
+		gf_pso_sprintf (pso, "end readonly def\n");
+		gf_pso_sprintf (pso, "FontName currentdict end definefont pop\n");
+	} else {
+		guchar *afm;
+		/* This is somewhat experimental - convert TTF to Type1 */
+		afm = ttf2pfa (pso->face->ft_face, embeddedname, pso->glyphs);
+		if (!afm) {
+			munmap (fbuf, st.st_size);
+			g_warning ("file %s: line %d: Cannot convert TTF %s to Type1", __FILE__, __LINE__, tt->ttf.name);
+			gff_pso_ensure_buffer_empty (pso);
+			return;
+		}
+		/* We take over ownership of afm here (be careful) */
+		pso->buf = afm;
+		pso->bufsize = strlen (afm);
+		pso->length = pso->bufsize;
 	}
-	gf_pso_sprintf (pso, "] def\n");
-	/* fixme: Use CID or something */
-	gf_pso_sprintf (pso, "/CharStrings %d dict dup begin\n", face->num_glyphs);
-	gf_pso_sprintf (pso, "/.notdef 0 def\n");
-	for (i = 1; i < face->num_glyphs; i++) {
-		gf_pso_sprintf (pso, "/g%d %d def\n", i, i);
-	}
-	gf_pso_sprintf (pso, "end readonly def\n");
-	gf_pso_sprintf (pso, "FontName currentdict end definefont pop\n");
 
 	/* Unmap font file */
 	munmap (fbuf, st.st_size);
@@ -1502,7 +1358,7 @@ gff_pso_from_truetype (const GnomeFontFace *face, GPFontEntry *entry)
 	/*
 	 * We have font downloaded (hopefully)
 	 *
-	 * With CharStrings: .nodef g1 g2 g3 g4 g5...
+	 * With CharStrings: .nodef _1 _2 _3
 	 *
 	 * Now we have to build usable 2-byte encoded font from it
 	 *
@@ -1510,7 +1366,7 @@ gff_pso_from_truetype (const GnomeFontFace *face, GPFontEntry *entry)
 
 	/* fixme: That is crap. We should use CID */
 	/* Bitch 'o' bitches (Lauris) ! */
-	if (face->num_glyphs < 256) {
+	if (pso->face->num_glyphs < 256) {
 		gint glyph;
 		/* 8-bit vector */
 		pso->encodedbytes = 1;
@@ -1519,11 +1375,11 @@ gff_pso_from_truetype (const GnomeFontFace *face, GPFontEntry *entry)
 		gf_pso_sprintf (pso, "/Encoding [\n");
 		for (glyph = 0; glyph < 256; glyph++) {
 			guint g;
-			g = (glyph < face->num_glyphs) ? glyph : 0;
-			if (g == 0) {
+			g = (glyph < pso->face->num_glyphs) ? glyph : 0;
+			if ((g == 0) || !PSO_GLYPH_MARKED (pso, glyph)) {
 				gf_pso_sprintf (pso, ((glyph & 0xf) == 0xf) ? "/.notdef\n" : "/.notdef ", g);
 			} else {
-				gf_pso_sprintf (pso, ((glyph & 0xf) == 0xf) ? "/g%d\n" : "/g%d ", g);
+				gf_pso_sprintf (pso, ((glyph & 0xf) == 0xf) ? "/_%d\n" : "/_%d ", g);
 			}
 		}
 		gf_pso_sprintf (pso, "] def currentdict end\n");
@@ -1532,7 +1388,7 @@ gff_pso_from_truetype (const GnomeFontFace *face, GPFontEntry *entry)
 		gint nfonts, nglyphs, i, j;
 		/* 16-bit vector */
 		pso->encodedbytes = 2;
-		nglyphs = face->num_glyphs;
+		nglyphs = pso->face->num_glyphs;
 		nfonts = (nglyphs + 255) >> 8;
 
 		gf_pso_sprintf (pso, "32 dict begin\n");
@@ -1553,10 +1409,10 @@ gff_pso_from_truetype (const GnomeFontFace *face, GPFontEntry *entry)
 				gint glyph;
 				glyph = 256 * i + j;
 				if (glyph >= nglyphs) glyph = 0;
-				if (glyph == 0) {
+				if ((glyph == 0) || !PSO_GLYPH_MARKED (pso, glyph)) {
 					gf_pso_sprintf (pso, ((j & 0xf) == 0xf) ? "/.notdef\n" : "/.notdef ");
 				} else {
-					gf_pso_sprintf (pso, ((j & 0xf) == 0xf) ? "/g%d\n" : "/g%d ", glyph);
+					gf_pso_sprintf (pso, ((j & 0xf) == 0xf) ? "/_%d\n" : "/_%d ", glyph);
 				}
 			}
 			gf_pso_sprintf (pso, "] def\n");
@@ -1573,52 +1429,36 @@ gff_pso_from_truetype (const GnomeFontFace *face, GPFontEntry *entry)
 		gf_pso_sprintf (pso, "currentdict end\n");
 		gf_pso_sprintf (pso, "/%s exch definefont pop\n", pso->encodedname);
 	}
-
-	return pso;
 }
 
-GFPSObject *
-gff_empty_pso (const GnomeFontFace *face, GPFontEntry *entry)
+static void
+gff_pso_ensure_buffer_empty (GFPSObject *pso)
 {
-	GFPSObject *pso;
-
-	/* Create PSO */
-	pso = g_new (GFPSObject, 1);
-	pso->bufsize = 1024;
 	pso->length = 0;
-	pso->buf = g_new (guchar, pso->bufsize);
-	pso->encodedname = g_strdup_printf ("GnomeEmpty-%s", entry->psname);
-	pso->encodedbytes = 1;
 
 	gf_pso_sprintf (pso, "%%Empty font generated by gnome-print\n");
-	gf_pso_sprintf (pso, "8 dict begin");
-	gf_pso_sprintf (pso, "/FontType 3 def\n");
-	gf_pso_sprintf (pso, "/FontMatrix [.001 0 0 .001 0 0] def\n");
-	gf_pso_sprintf (pso, "/FontBBox [0 0 750 950] def\n");
-	gf_pso_sprintf (pso, "/Encoding 256 array def\n");
-	gf_pso_sprintf (pso, "0 1 255 {Encoding exch /.notdef put} for\n");
-	gf_pso_sprintf (pso, "/CharProcs 2 dict def\n");
-	gf_pso_sprintf (pso, "CharProcs begin\n");
-	gf_pso_sprintf (pso, "/.notdef {\n");
+	gf_pso_sprintf (pso, "8 dict begin /FontType 3 def /FontMatrix [.001 0 0 .001 0 0] def /FontBBox [0 0 750 950] def\n");
+	gf_pso_sprintf (pso, "/Encoding 256 array def 0 1 255 {Encoding exch /.notdef put} for\n");
+	gf_pso_sprintf (pso, "/CharProcs 2 dict def CharProcs begin /.notdef {\n");
 	gf_pso_sprintf (pso, "0 0 moveto 750 0 lineto 750 950 lineto 0 950 lineto closepath\n");
 	gf_pso_sprintf (pso, "50 50 moveto 700 50 lineto 700 900 lineto 50 900 lineto closepath\n");
-	gf_pso_sprintf (pso, "eofill\n");
-	gf_pso_sprintf (pso, "} bind def\n");
-	gf_pso_sprintf (pso, "end\n");
-	gf_pso_sprintf (pso, "/BuildGlyph {\n");
-	gf_pso_sprintf (pso, "1000 0 0 0 750 950 setcachedevice\n");
-	gf_pso_sprintf (pso, "exch /CharProcs get exch\n");
-	gf_pso_sprintf (pso, "2 copy known not {pop /.notdef} if\n");
-	gf_pso_sprintf (pso, "get exec\n");
-	gf_pso_sprintf (pso, "} bind def\n");
-	gf_pso_sprintf (pso, "/BuildChar {1 index /Encoding get exch get\n");
-	gf_pso_sprintf (pso, "1 index /BuildGlyph get exec\n");
-	gf_pso_sprintf (pso, "} bind def\n");
-	gf_pso_sprintf (pso, "currentdict\n");
-	gf_pso_sprintf (pso, "end\n");
-	gf_pso_sprintf (pso, "/%s exch definefont pop\n", pso->encodedname);
-
-	return pso;
+	gf_pso_sprintf (pso, "eofill } bind def end\n");
+	gf_pso_sprintf (pso, "/BuildGlyph {1000 0 0 0 750 950 setcachedevice exch /CharProcs get exch\n");
+	gf_pso_sprintf (pso, "2 copy known not {pop /.notdef} if get exec } bind def\n");
+	gf_pso_sprintf (pso, "/BuildChar {1 index /Encoding get exch get 1 index /BuildGlyph get exec } bind def\n");
+	if (pso->encodedbytes == 1) {
+		/* 8-bit empty font */
+		gf_pso_sprintf (pso, "currentdict end /%s exch definefont pop\n", pso->encodedname);
+	} else {
+		/* 16-bit empty font */
+		gf_pso_sprintf (pso, "currentdict end /%s-Base exch definefont pop\n", pso->encodedname);
+		gf_pso_sprintf (pso, "32 dict begin /FontType 0 def /FontMatrix [1 0 0 1 0 0] def\n");
+		gf_pso_sprintf (pso, "/FontName /%s-Glyph-Composite def\n", pso->encodedname);
+		gf_pso_sprintf (pso, "/LanguageLevel 2 def /FMapType 2 def\n");
+		gf_pso_sprintf (pso, "/FDepVector [/%s-Base findfont] def", pso->encodedname);
+		gf_pso_sprintf (pso, "/Encoding 256 array def 0 1 255 {Encoding exch 0 put} for\n");
+		gf_pso_sprintf (pso, "currentdict end /%s exch definefont pop\n", pso->encodedname);
+	}
 }
 
 static void
@@ -1646,11 +1486,15 @@ gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...)
 }
 
 static void
-gf_pso_ensure_space (GFPSObject * pso, gint size)
+gf_pso_ensure_space (GFPSObject *pso, gint size)
 {
 	if (pso->length + size > pso->bufsize) {
-		while (pso->length + size > pso->bufsize) pso->bufsize <<= 1;
-		pso->buf = g_renew (guchar, pso->buf, pso->bufsize);
+		if (pso->bufsize < 1) {
+			pso->bufsize = MAX (size, 1024);
+			pso->buf = g_new (guchar, pso->bufsize);
+		} else {
+			while (pso->length + size > pso->bufsize) pso->bufsize <<= 1;
+			pso->buf = g_renew (guchar, pso->buf, pso->bufsize);
+		}
 	}
 }
-
