@@ -33,7 +33,7 @@
 
 #include <libart_lgpl/art_affine.h>
 
-#include "gpa-private.h"
+#include <gpa/gpa-private.h>
 #include "gnome-print-private.h"
 #include "gnome-print-meta.h"
 #include "gnome-print-multipage.h"
@@ -285,93 +285,6 @@ gnome_print_master_get_page_size_from_config (GnomePrintConfig *config, gdouble
 	return TRUE;
 }
 
-/**
- * gnome_print_master_set_paper:
- * @gpm: An initialised GnomePrintMaster.
- * @paper: Paper size required.
- * 
- * Set the paper size for the print.
- **/
-
-gint
-gnome_print_master_set_paper (GnomePrintMaster *gpm, const guchar *paper)
-{
-	gboolean ret;
-
-	g_return_val_if_fail (gpm != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (paper != NULL, GNOME_PRINT_ERROR_BADVALUE);
-
-	ret = gnome_print_config_set (gpm->config, GNOME_PRINT_KEY_PAPER_SIZE, paper);
-
-	return ret ? GNOME_PRINT_OK : GNOME_PRINT_ERROR_BADVALUE;
-}
-
-const guchar *
-gnome_print_master_get_paper (const GnomePrintMaster *gpm)
-{
-	g_return_val_if_fail (gpm != NULL, NULL);
-	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), NULL);
-
-	/* fixme: memleak */
-	return gnome_print_config_get (gpm->config, GNOME_PRINT_KEY_PAPER_SIZE);
-}
-
-/**
- * gnome_print_master_set_printer:
- * @gpm: An initialised GnomePrintMaster.
- * @printer: Printer device to use.
- * 
- * Sets physical printer used to print to.  If this is not set,
- * then a dialogue will be presented on each print.
- *
- * This functions takes ownership of the printer reference.
- **/
-
-gint
-gnome_print_master_set_printer (GnomePrintMaster *gpm, const guchar *printer)
-{
-	gboolean ret;
-
-	g_return_val_if_fail (gpm != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (printer != NULL, GNOME_PRINT_ERROR_BADVALUE);
-
-	ret = gnome_print_config_set (gpm->config, "Printer", printer);
-
-	return ret ? GNOME_PRINT_OK : GNOME_PRINT_ERROR_BADVALUE;
-}
-
-/**
- * gnome_print_master_set_copies:
- * @gpm: A GnomePrintContext.
- * @copies: Number of copies of each page to print.
- * @iscollate: Whether page copies are collated.
- * 
- * Set the number of copies to print, and whether pages are collated.
- * If @iscollate is set, then a multiple copy print to a physical
- * printer will order pages as "1,2,3,...1,2,3,..." otherwise they
- * will be ordered "1,1,...,2,2...".
- **/
-
-gint
-gnome_print_master_set_copies (GnomePrintMaster *gpm, gint copies, gboolean collate)
-{
-	gboolean ret;
-
-	g_return_val_if_fail (gpm != NULL, GNOME_PRINT_ERROR_UNKNOWN);
-	g_return_val_if_fail (GNOME_IS_PRINT_MASTER (gpm), GNOME_PRINT_ERROR_UNKNOWN);
-
-	/* fixme: key */
-	ret = gnome_print_config_set_int (gpm->config, GNOME_PRINT_KEY_NUM_COPIES, copies);
-	g_return_val_if_fail (ret, GNOME_PRINT_ERROR_UNKNOWN);
-	/* fixme: key */
-	ret = gnome_print_config_set_boolean (gpm->config, GNOME_PRINT_KEY_COLLATE, collate);
-	g_return_val_if_fail (ret, GNOME_PRINT_ERROR_UNKNOWN);
-
-	return GNOME_PRINT_OK;
-}
-
 /**
  * gnome_print_master_close:
  * @gpm: A GnomePrintMaster which has had printing performed
@@ -826,12 +739,17 @@ gpm_clear_config_data (GnomePrintMaster *gpm)
 	}
 }
 
+
+#ifdef __GNUC__
+#warning This function should not be here, we should not be including gpa-private.h too
+#endif
 /* All keys can be NULL */
-GnomePrintLayoutData *gnome_print_config_get_layout_data (GnomePrintConfig *config,
-							  const guchar *pagekey,
-							  const guchar *porientkey,
-							  const guchar *lorientkey,
-							  const guchar *layoutkey)
+GnomePrintLayoutData *
+gnome_print_config_get_layout_data (GnomePrintConfig *config,
+				    const guchar *pagekey,
+				    const guchar *porientkey,
+				    const guchar *lorientkey,
+				    const guchar *layoutkey)
 {
 	GnomePrintLayoutData *lyd;
 	guchar key[1024];
@@ -1105,9 +1023,6 @@ gpm_parse_transform (guchar *str, gdouble *transform)
 	}
 
 	memcpy (transform, t, 6 * sizeof (gdouble));
-#if 0	
-	PRINT_AFFINE ("Got affine:", transform);
-#endif	
 
 	return TRUE;
 }
