@@ -29,6 +29,22 @@
 
 G_BEGIN_DECLS
 
+/*
+ * GnomePrintMaster is frontend to GnomePrintContext creation and
+ * printing. Unlike contexts themselves, master cares about layout,
+ * number of copies and collate values. So it is usually safe bet to
+ * use it always.
+ *
+ * Notice, that page number(s) used by master are already adjusted
+ * to layout. I.e. if you print 2 pages per sheet, and invoke
+ * beginpage/showpage 4 times, gnome_print_master_get_pages gives
+ * you value 2.
+ *
+ * Also, if you wonder, what page dimensions you should be using in
+ * your application, use gnome_print_master_get_page_size_from_config.
+ *
+ */
+
 #define GNOME_TYPE_PRINT_MASTER (gnome_print_master_get_type ())
 #define GNOME_PRINT_MASTER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), GNOME_TYPE_PRINT_MASTER, GnomePrintMaster))
 #define GNOME_PRINT_MASTER_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), GNOME_TYPE_PRINT_MASTER, GnomePrintMasterClass))
@@ -55,6 +71,7 @@ GnomePrintConfig *gnome_print_master_get_config (GnomePrintMaster *gpm);
 GnomePrintContext *gnome_print_master_get_context (GnomePrintMaster *gpm);
 
 /* Signal GnomePrintMaster, that application has finished job */
+/* You should not close context given you by master, but use this instead */
 gint gnome_print_master_close (GnomePrintMaster *gpm);
 
 /* Send printout to printer, as defined by config */
@@ -72,15 +89,6 @@ gint gnome_print_master_get_pages (GnomePrintMaster *gpm);
 /* Sizes are given in PS points (GNOME_PRINT_PS_UNIT) */
 gboolean gnome_print_master_get_page_size_from_config (GnomePrintConfig *config, gdouble *width, gdouble *height);
 
-#if 0
-/* You really do not need these, use GnomePrintConfig instead */
-void gnome_print_master_set_paper (GnomePrintMaster *gpm, const GnomePaper *paper);
-const GnomePaper * gnome_print_master_get_paper (const GnomePrintMaster *gpm);
-
-void gnome_print_master_set_printer (GnomePrintMaster *gpm, GnomePrinter *printer);
-void gnome_print_master_set_copies  (GnomePrintMaster *gpm, gint copies, gboolean iscollate);
-#endif
-
 G_END_DECLS
 
 #endif /* GNOME_PRINT_MASTER_H */
