@@ -616,43 +616,6 @@ gpm_decode_pointer (const guchar *data, gpointer *ptr)
 	return decode_block (data, ptr, sizeof (*ptr));
 }
 
-#if 0
-static const char *
-decode_header (const char *data, GnomeMetaFileHeader *mh)
-{
-	data = decode_block (data, mh->signature, sizeof (mh->signature));
-	data = decode_int_header (data, &mh->size);
-	return data;
-}
-#endif
-
-#if 0
-static const char *
-decode_page_header (const char *data, GnomeMetaPageHeader *mh)
-{
-	data = decode_block (data, mh->signature, sizeof (mh->signature));
-	data = decode_int_header (data, &mh->size);
-	return data;
-}
-
-static const char *
-locate_page_header (const char *meta_stream, int page)
-{
-	const char *data = meta_stream + FILEHEADER_SIZE, *next;
-	GnomeMetaPageHeader page_header;
-
-	next = decode_page_header (data, &page_header);
-	while (page>0 && page_header.size != -1) {
-		data = next + page_header.size;
-		next = decode_page_header (data, &page_header);
-		page--;
-	}
-	if (page_header.size == -1)
-		return NULL;
-	return data;
-}
-#endif
-
 static gint
 gpm_render (GnomePrintContext *dest, const guchar *data, gint pos, gint len, gboolean pageops)
 {
@@ -973,143 +936,6 @@ gnome_print_meta_render_file_page (GnomePrintContext *ctx, const guchar *filenam
 	return ret;
 }
 
-#if 0
-/**
- * gnome_print_meta_render:
- * @destination: Destination printer context.
- * @meta_stream: a metadata stream to replay
- *
- * Plays the @meta_steam metadata stream into the @destination printer.
- *
- * Returns TRUE on success.
- */
-gboolean
-gnome_print_meta_render (GnomePrintContext *destination, const void *meta_stream)
-{
-	const char *data;
-	GnomeMetaFileHeader header;
-
-	g_return_val_if_fail (destination != NULL, FALSE);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (destination), FALSE);
-	g_return_val_if_fail (meta_stream != NULL, FALSE);
-
-	data = meta_stream;
-	data = decode_header (data, &header);
-	if (strncmp (header.signature, METAFILE_SIGNATURE, METAFILE_SIGNATURE_SIZE) != 0)
-		return FALSE;
-
-	if (header.size == -1){
-		g_warning ("This stream was not closed");
-		return FALSE;
-	}
-	return do_render (destination, data + PAGEHEADER_SIZE,
-			  header.size, -1);
-}
-
-/**
- * gnome_print_meta_render_from_object:
- * @destination: Destination printer context.
- * @source: an existing GnomePrintMeta printer context.
- *
- * Plays the contents of the GnomePrintMeta @source metadata stream
- * into the @destination printer.
- *
- * Returns: TRUE on success.
- */
-gboolean
-gnome_print_meta_render_from_object (GnomePrintContext *destination, const GnomePrintMeta *source)
-{
-#if 1
-	const char * data;
-	GnomeMetaFileHeader header;
-#endif
-	g_return_val_if_fail (destination != NULL, FALSE);
-	g_return_val_if_fail (source != NULL, FALSE);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (destination), FALSE);
-	g_return_val_if_fail (GNOME_IS_PRINT_META (source), FALSE);
-
-#if 0
-	return gnome_print_meta_render (destination, source->buf);
-#else
-	/* Render even unclosed metafiles */
-
-	data = source->buf;
-	data = decode_header (data, &header);
-
-	if (strncmp (header.signature, METAFILE_SIGNATURE, METAFILE_SIGNATURE_SIZE) != 0)
-		return FALSE;
-
-	return do_render (destination, data + PAGEHEADER_SIZE,
-			  source->b_length - FILEHEADER_SIZE - PAGEHEADER_SIZE, -1);
-#endif
-}
-
-/**
- * gnome_print_meta_render_page:
- * @destination: Destination printer context.
- * @meta_stream: a metadata stream to replay
- * @page: Page to be rendered.
- *
- * Plays the @meta_steam metadata stream into the @destination printer.
- * Output commands will only take place for page @page.
- *
- * Returns TRUE on success.
- */
-gboolean
-gnome_print_meta_render_page (GnomePrintContext *destination, const void *meta_stream, int page)
-{
-	const char *data;
-	GnomeMetaFileHeader header;
-	const char *page_data;
-
-	g_return_val_if_fail (destination != NULL, FALSE);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (destination), FALSE);
-	g_return_val_if_fail (meta_stream != NULL, FALSE);
-
-	data = meta_stream;
-	data = decode_header (data, &header);
-	if (strncmp (header.signature, METAFILE_SIGNATURE, METAFILE_SIGNATURE_SIZE) != 0)
-		return FALSE;
-
-	if (header.size == -1){
-		g_warning ("This printing context has not been closed");
-		return FALSE;
-	}
-
-	page_data = locate_page_header (meta_stream, page);
-	if (page_data == NULL) {
-		g_warning ("Trying to print a non-existant page");
-		return FALSE;
-	}
-	return do_render (destination, page_data + PAGEHEADER_SIZE,
-			  header.size, 1);
-}
-
-/**
- * gnome_print_meta_render_from_object_page:
- * @destination: Destination printer context.
- * @source: an existing GnomePrintMeta printer context.
- * @page: Page to be rendered.
- *
- * Plays the contents of the GnomePrintMeta @source metadata stream
- * into the @destination printer.  Output commands will only take place for page @page.
- *
- * Returns: TRUE on success.
- */
-gboolean
-gnome_print_meta_render_from_object_page (GnomePrintContext *destination,
-					  const GnomePrintMeta *source,
-					  int page)
-{
-	g_return_val_if_fail (destination != NULL, FALSE);
-	g_return_val_if_fail (source != NULL, FALSE);
-	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (destination), FALSE);
-	g_return_val_if_fail (GNOME_IS_PRINT_META (source), FALSE);
-
-	return gnome_print_meta_render_page (destination, source->buf, page);
-}
-#endif
-
 static void
 gpm_encode_string (GnomePrintContext *pc, const guchar *str)
 {
