@@ -1,32 +1,33 @@
-#define __GNOME_PRINT_PS2_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * gnome-print-ps2.c: A Postscript driver for GnomePrint based on
- * gnome-print-pdf which was based on the PS driver by Raph Levien.
+ *  gnome-print-ps2.c: A Postscript driver for GnomePrint
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Authors:
- *   Chema Celorio <chema@celorio.com>
- *   Lauris Kaplinski <lauris@helixcode.com>
+ *  Authors:
+ *    Chema Celorio <chema@celorio.com>
+ *    Lauris Kaplinski <lauris@helixcode.com>
  *
- * Copyright 2000-2001 Ximian, Inc. and authors
+ *  Copyright 2000-2001 Ximian, Inc. and authors
  *
+ *  References:
+ *    Document Structuring Conventions, Adobe. [http://partners.adobe.com/asn/developer/pdfs/tn/5001.DSC_Spec.pdf]
  */
 
+#define __GNOME_PRINT_PS2_C__
+
 #include <config.h>
 
 #include <math.h>
@@ -42,43 +43,42 @@
 #include <libgnomeprint/gnome-print-transport.h>
 #include <libgnomeprint/gnome-font-private.h>
 #include <libgnomeprint/gnome-print-encode-private.h>
-
 #include <libgnomeprint/gnome-pgl-private.h>
 
 #include "gnome-print-ps2.h"
 
 #define EOL "\n"
 
-typedef struct _GPPS2Font GPPS2Font;
-typedef struct _GPPS2Page GPPS2Page;
+typedef struct _GnomePrintPs2Font GnomePrintPs2Font;
+typedef struct _GnomePrintPs2Page GnomePrintPs2Page;
 
-struct _GPPS2Font {
-	GPPS2Font *next;
+struct _GnomePrintPs2Font {
+	GnomePrintPs2Font *next;
 	GnomeFontFace *face;
 	GFPSObject *pso;
 	gdouble currentsize;
 };
 
-struct _GPPS2Page {
-	GPPS2Page *next;
+struct _GnomePrintPs2Page {
+	GnomePrintPs2Page *next;
 	gchar *name;
 	gint number;
 	gboolean shown;
 	GSList *usedfonts;
 };
 
-struct _GnomePrintPS2 {
+struct _GnomePrintPs2 {
 	GnomePrintContext pc;
 
 	/* List of used fonts */
-	GPPS2Font *fonts;
+	GnomePrintPs2Font *fonts;
 	/* Active font */
-	GPPS2Font *selectedfont;
+	GnomePrintPs2Font *selectedfont;
 	/* Active color */
         gdouble r, g, b;
 	gint private_color_flag;
 	/* List of pages */
-	GPPS2Page *pages;
+	GnomePrintPs2Page *pages;
 	/* Bookkeeping */
 	gint gsave_level;
 	gint ps_level;
@@ -89,12 +89,12 @@ struct _GnomePrintPS2 {
 	gchar *bufname;
 };
 
-struct _GnomePrintPS2Class {
+struct _GnomePrintPs2Class {
 	GnomePrintContextClass parent_class;
 };
 
-static void gnome_print_ps2_class_init (GnomePrintPS2Class *klass);
-static void gnome_print_ps2_init (GnomePrintPS2 *PS2);
+static void gnome_print_ps2_class_init (GnomePrintPs2Class *klass);
+static void gnome_print_ps2_init (GnomePrintPs2 *PS2);
 static void gnome_print_ps2_finalize (GObject *object);
 
 static gint gnome_print_ps2_construct (GnomePrintContext *ctx);
@@ -109,17 +109,17 @@ static gint gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name
 static gint gnome_print_ps2_showpage (GnomePrintContext *pc);
 static gint gnome_print_ps2_close (GnomePrintContext *pc);
 
-static gint gp_ps2_set_color (GnomePrintPS2 *ps2);
-static gint gp_ps2_set_line (GnomePrintPS2 *ps2);
-static gint gp_ps2_set_dash (GnomePrintPS2 *ps2);
+static gint gnome_print_ps2_set_color (GnomePrintPs2 *ps2);
+static gint gnome_print_ps2_set_line (GnomePrintPs2 *ps2);
+static gint gnome_print_ps2_set_dash (GnomePrintPs2 *ps2);
 
-static gint gp_ps2_set_color_private (GnomePrintPS2 *ps2, gdouble r, gdouble g, gdouble b);
-static gint gp_ps2_set_font_private (GnomePrintPS2 *ps2, const GnomeFont *font);
+static gint gnome_print_ps2_set_color_private (GnomePrintPs2 *ps2, gdouble r, gdouble g, gdouble b);
+static gint gnome_print_ps2_set_font_private (GnomePrintPs2 *ps2, const GnomeFont *font);
 
-static gint gp_ps2_print_bpath (GnomePrintPS2 *ps2, const ArtBpath *bpath);
+static gint gnome_print_ps2_print_bpath (GnomePrintPs2 *ps2, const ArtBpath *bpath);
 
 static gchar *gnome_print_ps2_get_date (void);
-static gint gp_ps2_fprintf (GnomePrintPS2 *ps2, const char *format, ...);
+static gint gnome_print_ps2_fprintf (GnomePrintPs2 *ps2, const char *format, ...);
 
 static GnomePrintContextClass *parent_class;
 
@@ -129,21 +129,21 @@ gnome_print_ps2_get_type (void)
 	static GType ps2_type = 0;
 	if (!ps2_type) {
 		static const GTypeInfo ps2_info = {
-			sizeof (GnomePrintPS2Class),
+			sizeof (GnomePrintPs2Class),
 			NULL, NULL,
 			(GClassInitFunc) gnome_print_ps2_class_init,
 			NULL, NULL,
-			sizeof (GnomePrintPS2),
+			sizeof (GnomePrintPs2),
 			0,
 			(GInstanceInitFunc) gnome_print_ps2_init
 		};
-		ps2_type = g_type_register_static (GNOME_TYPE_PRINT_CONTEXT, "GnomePrintPS2", &ps2_info, 0);
+		ps2_type = g_type_register_static (GNOME_TYPE_PRINT_CONTEXT, "GnomePrintPs2", &ps2_info, 0);
 	}
 	return ps2_type;
 }
 
 static void
-gnome_print_ps2_class_init (GnomePrintPS2Class *klass)
+gnome_print_ps2_class_init (GnomePrintPs2Class *klass)
 {
 	GObjectClass *object_class;
 	GnomePrintContextClass *pc_class;
@@ -156,39 +156,34 @@ gnome_print_ps2_class_init (GnomePrintPS2Class *klass)
 	object_class->finalize = gnome_print_ps2_finalize;
 
 	pc_class->construct = gnome_print_ps2_construct;
-	pc_class->gsave = gnome_print_ps2_gsave;
-	pc_class->grestore = gnome_print_ps2_grestore;
-	pc_class->fill = gnome_print_ps2_fill;
-	pc_class->clip = gnome_print_ps2_clip;
-	pc_class->stroke = gnome_print_ps2_stroke;
-	pc_class->glyphlist = gnome_print_ps2_glyphlist;
-	pc_class->image = gnome_print_ps2_image;
 	pc_class->beginpage = gnome_print_ps2_beginpage;
 	pc_class->showpage = gnome_print_ps2_showpage;
+	pc_class->gsave = gnome_print_ps2_gsave;
+	pc_class->grestore = gnome_print_ps2_grestore;
+	pc_class->clip = gnome_print_ps2_clip;
+	pc_class->fill = gnome_print_ps2_fill;
+	pc_class->stroke = gnome_print_ps2_stroke;
+	pc_class->image = gnome_print_ps2_image;
+	pc_class->glyphlist = gnome_print_ps2_glyphlist;
 	pc_class->close = gnome_print_ps2_close;
 }
 
 static void
-gnome_print_ps2_init (GnomePrintPS2 *ps2)
+gnome_print_ps2_init (GnomePrintPs2 *ps2)
 {
 	ps2->ps_level = 2;
-	
 	ps2->gsave_level = 0;
-
 	ps2->fonts = NULL;
 	ps2->selectedfont = NULL;
-
 	ps2->private_color_flag = GP_GC_FLAG_UNSET;
-
 	ps2->pages = NULL;
-
 	ps2->buf = NULL;
 }
 
 static void
 gnome_print_ps2_finalize (GObject *object)
 {
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 
 	ps2 = GNOME_PRINT_PS2 (object);
 
@@ -205,10 +200,12 @@ gnome_print_ps2_finalize (GObject *object)
 	}
 
 	while (ps2->pages) {
-		GPPS2Page *p;
+		GnomePrintPs2Page *p;
 		p = ps2->pages;
-		if (!p->shown) g_warning ("Page %d %s was not shown", p->number, p->name);
-		if (p->name) g_free (p->name);
+		if (!p->shown)
+			g_warning ("Page %d %s was not shown", p->number, p->name);
+		if (p->name)
+			g_free (p->name);
 		while (p->usedfonts) {
 			p->usedfonts = g_slist_remove (p->usedfonts, p->usedfonts->data);
 		}
@@ -217,10 +214,12 @@ gnome_print_ps2_finalize (GObject *object)
 	}
 
 	while (ps2->fonts) {
-		GPPS2Font *f;
+		GnomePrintPs2Font *f;
 		f = ps2->fonts;
-		if (f->face) g_object_unref (G_OBJECT (f->face));
-		if (f->pso) gnome_font_face_pso_free (f->pso);
+		if (f->face)
+			g_object_unref (G_OBJECT (f->face));
+		if (f->pso)
+			gnome_font_face_pso_free (f->pso);
 		ps2->fonts = f->next;
 		g_free (f);
 	}
@@ -228,12 +227,10 @@ gnome_print_ps2_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-#define GPPS2_BIGVAL 1e18
-
 static gint
 gnome_print_ps2_construct (GnomePrintContext *ctx)
 {
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 	gchar *tmp;
 	gint ret, fd;
 	const GnomePrintUnit *unit;
@@ -284,19 +281,19 @@ gnome_print_ps2_construct (GnomePrintContext *ctx)
 static gint
 gnome_print_ps2_gsave (GnomePrintContext *ctx)
 {
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 
 	ps2 = GNOME_PRINT_PS2 (ctx);
 
 	ps2->gsave_level += 1;
 
-	return gp_ps2_fprintf (ps2, "q" EOL);
+	return gnome_print_ps2_fprintf (ps2, "q" EOL);
 }
 
 static gint
 gnome_print_ps2_grestore (GnomePrintContext *ctx)
 {
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 
 	ps2 = GNOME_PRINT_PS2 (ctx);
 
@@ -304,24 +301,24 @@ gnome_print_ps2_grestore (GnomePrintContext *ctx)
 	ps2->selectedfont = NULL;
 	ps2->private_color_flag = GP_GC_FLAG_UNSET;
 
-	return gp_ps2_fprintf (ps2, "Q" EOL);
+	return gnome_print_ps2_fprintf (ps2, "Q" EOL);
 }
 
 static gint
 gnome_print_ps2_clip (GnomePrintContext *ctx, const ArtBpath *bpath, ArtWindRule rule)
 {
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 	gint ret;
 
 	ps2 = GNOME_PRINT_PS2 (ctx);
 
-	ret = gp_ps2_print_bpath (ps2, bpath);
+	ret = gnome_print_ps2_print_bpath (ps2, bpath);
 	g_return_val_if_fail (ret >= 0, ret);
 
 	if (rule == ART_WIND_RULE_NONZERO) {
-		ret = gp_ps2_fprintf (ps2, "W" EOL);
+		ret = gnome_print_ps2_fprintf (ps2, "W" EOL);
 	} else {
-		ret = gp_ps2_fprintf (ps2, "W*" EOL);
+		ret = gnome_print_ps2_fprintf (ps2, "W*" EOL);
 	}
 
 	return ret;
@@ -330,21 +327,21 @@ gnome_print_ps2_clip (GnomePrintContext *ctx, const ArtBpath *bpath, ArtWindRule
 static gint
 gnome_print_ps2_fill (GnomePrintContext *ctx, const ArtBpath *bpath, ArtWindRule rule)
 {
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 	gint ret;
 
 	ps2 = GNOME_PRINT_PS2 (ctx);
 
-	ret = gp_ps2_set_color (ps2);
+	ret = gnome_print_ps2_set_color (ps2);
 	g_return_val_if_fail (ret >= 0, ret);
 
-	ret = gp_ps2_print_bpath (ps2, bpath);
+	ret = gnome_print_ps2_print_bpath (ps2, bpath);
 	g_return_val_if_fail (ret >= 0, ret);
 
 	if (rule == ART_WIND_RULE_NONZERO) {
-		ret = gp_ps2_fprintf (ps2, "f" EOL);
+		ret = gnome_print_ps2_fprintf (ps2, "f" EOL);
 	} else {
-		ret = gp_ps2_fprintf (ps2, "f*" EOL);
+		ret = gnome_print_ps2_fprintf (ps2, "f*" EOL);
 	}
 
 	/* Update bbox */
@@ -356,22 +353,22 @@ gnome_print_ps2_fill (GnomePrintContext *ctx, const ArtBpath *bpath, ArtWindRule
 static gint
 gnome_print_ps2_stroke (GnomePrintContext *ctx, const ArtBpath *bpath)
 {
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 	gint ret;
 
 	ps2 = GNOME_PRINT_PS2 (ctx);
 
-	ret = gp_ps2_set_color (ps2);
+	ret = gnome_print_ps2_set_color (ps2);
 	g_return_val_if_fail (ret >= 0, ret);
-	ret = gp_ps2_set_line (ps2);
+	ret = gnome_print_ps2_set_line (ps2);
 	g_return_val_if_fail (ret >= 0, ret);
-	ret = gp_ps2_set_dash (ps2);
+	ret = gnome_print_ps2_set_dash (ps2);
 	g_return_val_if_fail (ret >= 0, ret);
 
-	ret = gp_ps2_print_bpath (ps2, bpath);
+	ret = gnome_print_ps2_print_bpath (ps2, bpath);
 	g_return_val_if_fail (ret >= 0, ret);
 
-	ret = gp_ps2_fprintf (ps2, "S" EOL);
+	ret = gnome_print_ps2_fprintf (ps2, "S" EOL);
 
 	return ret;
 }
@@ -379,7 +376,7 @@ gnome_print_ps2_stroke (GnomePrintContext *ctx, const ArtBpath *bpath)
 static int
 gnome_print_ps2_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *px, gint w, gint h, gint rowstride, gint ch)
 {
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 	gint ret;
 	gchar *hex;
 	gint hex_size;
@@ -387,23 +384,23 @@ gnome_print_ps2_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 
 	ps2 = GNOME_PRINT_PS2 (pc);
 
-	ret = gp_ps2_fprintf (ps2, "q" EOL);
+	ret = gnome_print_ps2_fprintf (ps2, "q" EOL);
 	g_return_val_if_fail (ret >= 0, ret);
-	ret = gp_ps2_fprintf (ps2, "[%g %g %g %g %g %g]cm" EOL, ctm[0], ctm[1], ctm[2], ctm[3], ctm[4], ctm[5]);
+	ret = gnome_print_ps2_fprintf (ps2, "[%g %g %g %g %g %g]cm" EOL, ctm[0], ctm[1], ctm[2], ctm[3], ctm[4], ctm[5]);
 	g_return_val_if_fail (ret >= 0, ret);
 
 	/* Image commands */
-	ret = gp_ps2_fprintf (ps2, "/buf %d string def" EOL "%d %d 8" EOL, w * ch, w, h);
+	ret = gnome_print_ps2_fprintf (ps2, "/buf %d string def" EOL "%d %d 8" EOL, w * ch, w, h);
 	g_return_val_if_fail (ret >= 0, ret);
-	ret = gp_ps2_fprintf (ps2, "[%d 0 0 %d 0 %d]" EOL, w, -h, h);
+	ret = gnome_print_ps2_fprintf (ps2, "[%d 0 0 %d 0 %d]" EOL, w, -h, h);
 	g_return_val_if_fail (ret >= 0, ret);
-	ret = gp_ps2_fprintf (ps2, "{ currentfile buf readhexstring pop }\n");
+	ret = gnome_print_ps2_fprintf (ps2, "{ currentfile buf readhexstring pop }\n");
 	g_return_val_if_fail (ret >= 0, ret);
 
 	if (ch == 1) {
-		ret = gp_ps2_fprintf (ps2, "image" EOL);
+		ret = gnome_print_ps2_fprintf (ps2, "image" EOL);
 	} else {
-		ret = gp_ps2_fprintf (ps2, "false %d colorimage" EOL, ch);
+		ret = gnome_print_ps2_fprintf (ps2, "false %d colorimage" EOL, ch);
 	}
 	g_return_val_if_fail (ret >= 0, ret);
 
@@ -413,13 +410,13 @@ gnome_print_ps2_image (GnomePrintContext *pc, const gdouble *ctm, const guchar *
 		hex_size = gnome_print_encode_hex (px + r * rowstride, hex, w * ch);
 		ret = fwrite (hex, sizeof (gchar), hex_size, ps2->buf);
 		g_return_val_if_fail (ret >= 0, ret);
-		ret = gp_ps2_fprintf (ps2, EOL);
+		ret = gnome_print_ps2_fprintf (ps2, EOL);
 		g_return_val_if_fail (ret >= 0, ret);
 	}
 
 	g_free (hex);
 	
-	ret = gp_ps2_fprintf (ps2, "Q" EOL);
+	ret = gnome_print_ps2_fprintf (ps2, "Q" EOL);
 	g_return_val_if_fail (ret >= 0, ret);
 
 	return GNOME_PRINT_OK;
@@ -431,20 +428,20 @@ static gint
 gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphList *gl)
 {
 	static gdouble id[] = {1.0, 0.0, 0.0, 1.0, 0.0, 0.0};
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 	GnomePosGlyphList *pgl;
 	gboolean identity;
 	gdouble dx, dy;
 	gint ret, s;
 
-	ps2 = (GnomePrintPS2 *) pc;
+	ps2 = (GnomePrintPs2 *) pc;
 
 	identity = ((fabs (a[0] - 1.0) < EPSILON) && (fabs (a[1]) < EPSILON) && (fabs (a[2]) < EPSILON) && (fabs (a[3] - 1.0) < EPSILON));
 
 	if (!identity) {
-		ret = gp_ps2_fprintf (ps2, "q" EOL);
+		ret = gnome_print_ps2_fprintf (ps2, "q" EOL);
 		g_return_val_if_fail (ret >= 0, ret);
-		ret = gp_ps2_fprintf (ps2, "[%g %g %g %g %g %g]cm" EOL, a[0], a[1], a[2], a[3], a[4], a[5]);
+		ret = gnome_print_ps2_fprintf (ps2, "[%g %g %g %g %g %g]cm" EOL, a[0], a[1], a[2], a[3], a[4], a[5]);
 		g_return_val_if_fail (ret >= 0, ret);
 		dx = dy = 0.0;
 	} else {
@@ -460,20 +457,20 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 
 		ps = pgl->strings + s;
 
-		ret = gp_ps2_set_font_private (ps2, gnome_rfont_get_font (ps->rfont));
+		ret = gnome_print_ps2_set_font_private (ps2, gnome_rfont_get_font (ps->rfont));
 		g_return_val_if_fail (ret >= 0, ret);
-		ret = gp_ps2_set_color_private (ps2,
+		ret = gnome_print_ps2_set_color_private (ps2,
 						((ps->color >> 24) & 0xff) / 255.0,
 						((ps->color >> 16) & 0xff) / 255.0,
 						((ps->color >>  8) & 0xff) / 255.0);
 		g_return_val_if_fail (ret >= 0, ret);
-		ret = gp_ps2_fprintf (ps2, "%g %g m" EOL, pgl->glyphs[ps->start].x + dx, pgl->glyphs[ps->start].y + dy);
+		ret = gnome_print_ps2_fprintf (ps2, "%g %g m" EOL, pgl->glyphs[ps->start].x + dx, pgl->glyphs[ps->start].y + dy);
 		g_return_val_if_fail (ret >= 0, ret);
 		/* Build string */
 
 
 
-		ret = gp_ps2_fprintf (ps2, "(");
+		ret = gnome_print_ps2_fprintf (ps2, "(");
 
 		if (ps2->selectedfont->pso->encodedbytes == 1) {
 			/* 8-bit encoding */
@@ -481,7 +478,7 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 				gint glyph;
 				glyph = pgl->glyphs[i].glyph & 0xff;
 				gnome_font_face_pso_mark_glyph (ps2->selectedfont->pso, glyph);
-				ret = gp_ps2_fprintf (ps2, "\\%o", glyph);
+				ret = gnome_print_ps2_fprintf (ps2, "\\%o", glyph);
 				g_return_val_if_fail (ret >= 0, ret);
 			}
 		} else {
@@ -491,30 +488,30 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 				gnome_font_face_pso_mark_glyph (ps2->selectedfont->pso, pgl->glyphs[i].glyph);
 				glyph = pgl->glyphs[i].glyph & 0xff;
 				page = (pgl->glyphs[i].glyph >> 8) & 0xff;
-				ret = gp_ps2_fprintf (ps2, "\\%o\\%o", page, glyph);
+				ret = gnome_print_ps2_fprintf (ps2, "\\%o\\%o", page, glyph);
 				g_return_val_if_fail (ret >= 0, ret);
 			}
 		}
-		ret = gp_ps2_fprintf (ps2, ")" EOL);
+		ret = gnome_print_ps2_fprintf (ps2, ")" EOL);
 		g_return_val_if_fail (ret >= 0, ret);
 		/* Build array */
-		ret = gp_ps2_fprintf (ps2, "[");
+		ret = gnome_print_ps2_fprintf (ps2, "[");
 		g_return_val_if_fail (ret >= 0, ret);
 		for (i = ps->start + 1; i < ps->start + ps->length; i++) {
-			ret = gp_ps2_fprintf (ps2, "%g %g ",
+			ret = gnome_print_ps2_fprintf (ps2, "%g %g ",
 					      pgl->glyphs[i].x - pgl->glyphs[i-1].x,
 					      pgl->glyphs[i].y - pgl->glyphs[i-1].y);
 			g_return_val_if_fail (ret >= 0, ret);
 		}
-		ret = gp_ps2_fprintf (ps2, "0 0] ");
+		ret = gnome_print_ps2_fprintf (ps2, "0 0] ");
 		g_return_val_if_fail (ret >= 0, ret);
 		/* xyshow */
-		ret = gp_ps2_fprintf (ps2, "xyshow" EOL);
+		ret = gnome_print_ps2_fprintf (ps2, "xyshow" EOL);
 		g_return_val_if_fail (ret >= 0, ret);
 	}
 
 	if (!identity) {
-		ret = gp_ps2_fprintf (ps2, "Q" EOL);
+		ret = gnome_print_ps2_fprintf (ps2, "Q" EOL);
 		g_return_val_if_fail (ret >= 0, ret);
 		ps2->selectedfont = NULL;
 		ps2->private_color_flag = GP_GC_FLAG_UNSET;
@@ -528,8 +525,8 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 static gint
 gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 {
-	GnomePrintPS2 *ps2;
-	GPPS2Page *p;
+	GnomePrintPs2 *ps2;
+	GnomePrintPs2Page *p;
 	gint number;
 	gint ret;
 
@@ -537,7 +534,7 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 
 	number = ps2->pages ? ps2->pages->number : 0;
 
-	p = g_new (GPPS2Page, 1);
+	p = g_new (GnomePrintPs2Page, 1);
 	p->next = ps2->pages;
 	p->name = g_strdup (name);
 	p->number = number + 1;
@@ -549,9 +546,9 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 	ps2->selectedfont = NULL;
 	ps2->private_color_flag = GP_GC_FLAG_UNSET;
 
-	ret = gp_ps2_fprintf (ps2, "%%%%Page: %s %d" EOL, name, p->number);
+	ret = gnome_print_ps2_fprintf (ps2, "%%%%Page: %s %d" EOL, name, p->number);
 	g_return_val_if_fail (ret >= 0, ret);
-	ret = gp_ps2_fprintf (ps2, "%%%%PageResources: (atend)" EOL);
+	ret = gnome_print_ps2_fprintf (ps2, "%%%%PageResources: (atend)" EOL);
 	g_return_val_if_fail (ret >= 0, ret);
 
 	ret = gnome_print_newpath (pc);
@@ -580,25 +577,26 @@ gnome_print_ps2_beginpage (GnomePrintContext *pc, const guchar *name)
 static gint
 gnome_print_ps2_showpage (GnomePrintContext *pc)
 {
-	GnomePrintPS2 *ps2;
+	GnomePrintPs2 *ps2;
 	gint ret;
 
 	ps2 = GNOME_PRINT_PS2 (pc);
 
-	if (ps2->pages) ps2->pages->shown = TRUE;
+	if (ps2->pages)
+		ps2->pages->shown = TRUE;
 	ps2->selectedfont = NULL;
 	ps2->private_color_flag = GP_GC_FLAG_UNSET;
 
-	ret = gp_ps2_fprintf (ps2, "SP" EOL);
+	ret = gnome_print_ps2_fprintf (ps2, "SP" EOL);
 	g_return_val_if_fail (ret >= 0, ret);
-	ret = gp_ps2_fprintf (ps2, "%%%%PageTrailer" EOL);
+	ret = gnome_print_ps2_fprintf (ps2, "%%%%PageTrailer" EOL);
 	g_return_val_if_fail (ret >= 0, ret);
-	ret = gp_ps2_fprintf (ps2, "%%%%PageResources: procset gnome-print-procs-%s" EOL, VERSION);
+	ret = gnome_print_ps2_fprintf (ps2, "%%%%PageResources: procset gnome-print-procs-%s" EOL, VERSION);
 	g_return_val_if_fail (ret >= 0, ret);
 	while (ps2->pages->usedfonts) {
-		GPPS2Font *font;
+		GnomePrintPs2Font *font;
 		font = ps2->pages->usedfonts->data;
-		ret = gp_ps2_fprintf (ps2, "%%%%+ font %s" EOL, font->pso->encodedname);
+		ret = gnome_print_ps2_fprintf (ps2, "%%%%+ font %s" EOL, font->pso->encodedname);
 		g_return_val_if_fail (ret >= 0, ret);
 		ps2->pages->usedfonts = g_slist_remove (ps2->pages->usedfonts, ps2->pages->usedfonts->data);
 	}
@@ -609,9 +607,11 @@ gnome_print_ps2_showpage (GnomePrintContext *pc)
 static gint
 gnome_print_ps2_close (GnomePrintContext *pc)
 {
-	GnomePrintPS2 *ps2;
-	GPPS2Font *f;
+	GnomePrintPs2 *ps2;
+	GnomePrintPs2Font *f;
 	gchar *date;
+	gchar *orientation;
+	gchar *name;
 	gint len, ret;
 
 	ps2 = GNOME_PRINT_PS2 (pc);
@@ -636,17 +636,39 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	gnome_print_transport_printf (pc->transport, "%%%%LanguageLevel: 2" EOL);
 	gnome_print_transport_printf (pc->transport, "%%%%DocumentMedia: Regular %d %d 0 () ()" EOL,
 				      (gint) (ps2->bbox.x1 - ps2->bbox.x0), (gint) (ps2->bbox.y1 - ps2->bbox.y0));
-	gnome_print_transport_printf (pc->transport, "%%%%Orientation: %s" EOL,
-				      ((ps2->bbox.x1 - ps2->bbox.x0) <= (ps2->bbox.y1 - ps2->bbox.y0)) ? "Portrait" : "Landscape");
+
+	/* Orientation */
+	gnome_print_transport_printf (pc->transport, "%%%%Orientation: ");
+	orientation = gnome_print_config_get (pc->config, GNOME_PRINT_KEY_ORIENTATION);
+	if (strcmp ("R0", orientation) == 0) {
+		gnome_print_transport_printf (pc->transport, "Portrait");
+	} else if (strcmp ("R90", orientation) == 0) {
+		gnome_print_transport_printf (pc->transport, "Landscape");
+	} else if (strcmp ("R180", orientation) == 0) {
+		gnome_print_transport_printf (pc->transport, "Reverse Portrait");
+	} else if (strcmp ("R270", orientation) == 0) {
+		gnome_print_transport_printf (pc->transport, "Reverse Landscape");
+	} else {
+		g_warning ("Could not interpret Orientation from GnomePrintConfig [%s]<-\n", orientation);
+	}
+	g_free (orientation);
+	gnome_print_transport_printf (pc->transport, EOL);
+	/* End: Orientation */
+
 	gnome_print_transport_printf (pc->transport, "%%%%BoundingBox: %d %d %d %d" EOL,
 				      (gint) floor (ps2->bbox.x0),
 				      (gint) floor (ps2->bbox.y0),
 				      (gint) ceil (ps2->bbox.x1),
 				      (gint) ceil (ps2->bbox.y1));
 	gnome_print_transport_printf (pc->transport, "%%%%Pages: %d" EOL, ps2->pages->number);
-	/* fixme: Orientation: */
 	gnome_print_transport_printf (pc->transport, "%%%%PageOrder: Ascend" EOL);
-	gnome_print_transport_printf (pc->transport, "%%%%Title: %s" EOL, "Document Title goes here");
+
+	name = gnome_print_config_get (pc->config,  GNOME_PRINT_KEY_DOCUMENT_NAME);
+	if (name) {
+		gnome_print_transport_printf (pc->transport, "%%%%Title: %s" EOL, name);
+		g_free (name);
+	}
+	
 	gnome_print_transport_printf (pc->transport, "%%%%DocumentSuppliedResources: procset pnome-print-procs-%s" EOL, VERSION);
 	/* %%DocumentSuppliedResources: */
 	for (f = ps2->fonts; f != NULL; f = f->next) {
@@ -678,6 +700,12 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 	gnome_print_transport_printf (pc->transport, "%%%%EndProlog" EOL);
 	/* Setup */
 	gnome_print_transport_printf (pc->transport, "%%%%BeginSetup" EOL);
+	/* setpagedevice*/
+	gnome_print_transport_printf (pc->transport, "<<" EOL);
+	gnome_print_transport_printf (pc->transport, "/PageSize [%d %d]" EOL, (gint) (ps2->bbox.x1 - ps2->bbox.x0), (gint) (ps2->bbox.y1 - ps2->bbox.y0));
+	gnome_print_transport_printf (pc->transport, "/ImagingBBox null" EOL);
+	gnome_print_transport_printf (pc->transport, "/Orientation %d" EOL, 0); /* FIXME: real orientation */
+	gnome_print_transport_printf (pc->transport, ">> setpagedevice" EOL);
 	/* Download fonts */
 	for (f = ps2->fonts; f != NULL; f = f->next) {
 		gnome_font_face_pso_ensure_buffer (f->pso);
@@ -713,26 +741,27 @@ gnome_print_ps2_close (GnomePrintContext *pc)
 }
 
 static gint
-gp_ps2_set_color (GnomePrintPS2 *ps2)
+gnome_print_ps2_set_color (GnomePrintPs2 *ps2)
 {
 	GnomePrintContext *ctx;
 
 	ctx = GNOME_PRINT_CONTEXT (ps2);
 
-	return gp_ps2_set_color_private (ps2, gp_gc_get_red (ctx->gc), gp_gc_get_green (ctx->gc), gp_gc_get_blue (ctx->gc));
+	return gnome_print_ps2_set_color_private (ps2, gp_gc_get_red (ctx->gc), gp_gc_get_green (ctx->gc), gp_gc_get_blue (ctx->gc));
 }
 
 static gint
-gp_ps2_set_line (GnomePrintPS2 *ps2)
+gnome_print_ps2_set_line (GnomePrintPs2 *ps2)
 {
 	GnomePrintContext *ctx;
 	gint ret;
 
 	ctx = GNOME_PRINT_CONTEXT (ps2);
 
-	if (gp_gc_get_line_flag (ctx->gc) == GP_GC_FLAG_CLEAR) return 0;
+	if (gp_gc_get_line_flag (ctx->gc) == GP_GC_FLAG_CLEAR)
+		return 0;
 
-	ret = gp_ps2_fprintf (ps2, "%g w %i J %i j %g M" EOL,
+	ret = gnome_print_ps2_fprintf (ps2, "%g w %i J %i j %g M" EOL,
 			      gp_gc_get_linewidth (ctx->gc),
 			      gp_gc_get_linecap (ctx->gc),
 			      gp_gc_get_linejoin (ctx->gc),
@@ -743,7 +772,7 @@ gp_ps2_set_line (GnomePrintPS2 *ps2)
 }
 
 static gint
-gp_ps2_set_dash (GnomePrintPS2 *ps2)
+gnome_print_ps2_set_dash (GnomePrintPs2 *ps2)
 {
 	GnomePrintContext *ctx;
 	const ArtVpathDash *dash;
@@ -751,28 +780,30 @@ gp_ps2_set_dash (GnomePrintPS2 *ps2)
 
 	ctx = GNOME_PRINT_CONTEXT (ps2);
 
-	if (gp_gc_get_dash_flag (ctx->gc) == GP_GC_FLAG_CLEAR) return 0;
+	if (gp_gc_get_dash_flag (ctx->gc) == GP_GC_FLAG_CLEAR)
+		return 0;
 
 	dash = gp_gc_get_dash (ctx->gc);
-	ret = gp_ps2_fprintf (ps2, "[");
-	for (i = 0; i < dash->n_dash; i++) ret += gp_ps2_fprintf (ps2, " %g", dash->dash[i]);
-	ret += gp_ps2_fprintf (ps2, "]%g d" EOL, dash->n_dash > 0 ? dash->offset : 0.0);
+	ret = gnome_print_ps2_fprintf (ps2, "[");
+	for (i = 0; i < dash->n_dash; i++) ret += gnome_print_ps2_fprintf (ps2, " %g", dash->dash[i]);
+	ret += gnome_print_ps2_fprintf (ps2, "]%g d" EOL, dash->n_dash > 0 ? dash->offset : 0.0);
 	gp_gc_set_dash_flag (ctx->gc, GP_GC_FLAG_CLEAR);
 
 	return ret;
 }
 
 static gint
-gp_ps2_set_color_private (GnomePrintPS2 *ps2, gdouble r, gdouble g, gdouble b)
+gnome_print_ps2_set_color_private (GnomePrintPs2 *ps2, gdouble r, gdouble g, gdouble b)
 {
 	GnomePrintContext *ctx;
 	gint ret;
 
 	ctx = GNOME_PRINT_CONTEXT (ps2);
 
-	if ((ps2->private_color_flag == GP_GC_FLAG_CLEAR) && (r == ps2->r) && (g == ps2->g) && (b == ps2->b)) return 0;
+	if ((ps2->private_color_flag == GP_GC_FLAG_CLEAR) && (r == ps2->r) && (g == ps2->g) && (b == ps2->b))
+		return 0;
 
-	ret = gp_ps2_fprintf (ps2, "%.3g %.3g %.3g rg" EOL, r, g, b);
+	ret = gnome_print_ps2_fprintf (ps2, "%.3g %.3g %.3g rg" EOL, r, g, b);
 
 	ps2->r = r;
 	ps2->g = g;
@@ -783,23 +814,25 @@ gp_ps2_set_color_private (GnomePrintPS2 *ps2, gdouble r, gdouble g, gdouble b)
 }
 
 static gint
-gp_ps2_set_font_private (GnomePrintPS2 *ps2, const GnomeFont *font)
+gnome_print_ps2_set_font_private (GnomePrintPs2 *ps2, const GnomeFont *font)
 {
 	const GnomeFontFace *face;
-	GPPS2Font *f;
+	GnomePrintPs2Font *f;
 	gint ret;
 	GSList *l;
 
-	if (ps2->selectedfont && (ps2->selectedfont->face == font->face) && (ps2->selectedfont->currentsize = font->size)) return 0;
+	if (ps2->selectedfont && (ps2->selectedfont->face == font->face) && (ps2->selectedfont->currentsize = font->size))
+		return 0;
 
 	face = gnome_font_get_face (font);
 
 	for (f = ps2->fonts; f != NULL; f = f->next) {
-		if (f->face == face) break;
+		if (f->face == face)
+			break;
 	}
 	if (!f) {
 		/* No entry, so create one */
-		f = g_new (GPPS2Font, 1);
+		f = g_new (GnomePrintPs2Font, 1);
 		f->next = ps2->fonts;
 		ps2->fonts = f;
 		f->face = (GnomeFontFace *) face;
@@ -809,13 +842,14 @@ gp_ps2_set_font_private (GnomePrintPS2 *ps2, const GnomeFont *font)
 	}
 
 	for (l = ps2->pages->usedfonts; l != NULL; l = l->next) {
-		if ((GPPS2Font *) l->data == f) break;
+		if ((GnomePrintPs2Font *) l->data == f)
+			break;
 	}
 	if (!l) {
 		ps2->pages->usedfonts = g_slist_prepend (ps2->pages->usedfonts, f);
 	}
 
-	ret = gp_ps2_fprintf (ps2, "/%s FF %g F" EOL, f->pso->encodedname, gnome_font_get_size (font));
+	ret = gnome_print_ps2_fprintf (ps2, "/%s FF %g F" EOL, f->pso->encodedname, gnome_font_get_size (font));
 
 	f->currentsize = font->size;
 	ps2->selectedfont = f;
@@ -844,33 +878,35 @@ gnome_print_ps2_new (GnomePrintConfig *config)
 }
 
 static gint
-gp_ps2_print_bpath (GnomePrintPS2 *ps2, const ArtBpath *bpath)
+gnome_print_ps2_print_bpath (GnomePrintPs2 *ps2, const ArtBpath *bpath)
 {
 	gboolean started, closed;
 	
-	gp_ps2_fprintf (ps2, "n" EOL);
+	gnome_print_ps2_fprintf (ps2, "n" EOL);
 
 	started = FALSE;
 	closed = FALSE;
 	while (bpath->code != ART_END) {
 		switch (bpath->code) {
 		case ART_MOVETO_OPEN:
-			if (started && closed) gp_ps2_fprintf (ps2, "h" EOL);
+			if (started && closed)
+				gnome_print_ps2_fprintf (ps2, "h" EOL);
 			closed = FALSE;
 			started = FALSE;
-			gp_ps2_fprintf (ps2, "%g %g m" EOL, bpath->x3, bpath->y3);
+			gnome_print_ps2_fprintf (ps2, "%g %g m" EOL, bpath->x3, bpath->y3);
 			break;
 		case ART_MOVETO:
-			if (started && closed) gp_ps2_fprintf (ps2, "h" EOL);
+			if (started && closed)
+				gnome_print_ps2_fprintf (ps2, "h" EOL);
 			closed = TRUE;
 			started = TRUE;
-			gp_ps2_fprintf (ps2, "%g %g m" EOL, bpath->x3, bpath->y3);
+			gnome_print_ps2_fprintf (ps2, "%g %g m" EOL, bpath->x3, bpath->y3);
 			break;
 		case ART_LINETO:
-			gp_ps2_fprintf (ps2, "%g %g l" EOL, bpath->x3, bpath->y3);
+			gnome_print_ps2_fprintf (ps2, "%g %g l" EOL, bpath->x3, bpath->y3);
 			break;
 		case ART_CURVETO:
-			gp_ps2_fprintf (ps2, "%g %g %g %g %g %g c" EOL,
+			gnome_print_ps2_fprintf (ps2, "%g %g %g %g %g %g c" EOL,
 					bpath->x1, bpath->y1,
 					bpath->x2, bpath->y2,
 					bpath->x3, bpath->y3);
@@ -882,7 +918,8 @@ gp_ps2_print_bpath (GnomePrintPS2 *ps2, const ArtBpath *bpath)
 		bpath += 1;
 	}
 
-	if (started && closed) gp_ps2_fprintf (ps2, "h" EOL);
+	if (started && closed)
+		gnome_print_ps2_fprintf (ps2, "h" EOL);
 	
 	return 0;
 }
@@ -923,7 +960,7 @@ gnome_print_ps2_get_date (void)
 }
 
 static gint
-gp_ps2_fprintf (GnomePrintPS2 *ps2, const char *format, ...)
+gnome_print_ps2_fprintf (GnomePrintPs2 *ps2, const char *format, ...)
 {
  	va_list arguments;
  	gchar *text;
