@@ -1,30 +1,31 @@
-#define __GPA_LIST_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * This file is part of libgnomeprint 2
+ *  gpa-list.c: 
  *
- * Libgnomeprint is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
- * Libgnomeprint is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with the libgnomeprint; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Authors :
- *   Lauris Kaplinski <lauris@ximian.com>
+ *  Authors :
+ *    Jose M. Celorio <chema@ximian.com>
+ *    Lauris Kaplinski <lauris@ximian.com>
  *
- * Copyright (C) 2001 Ximian, Inc.
+ *  Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
  *
  */
 
+#define __GPA_LIST_C__
+
 #include <string.h>
 #include "gpa-utils.h"
 #include "gpa-reference.h"
@@ -272,43 +273,3 @@ gpa_list_add_child (GPAList *list, GPANode *child, GPANode *ref)
 
 	return TRUE;
 }
-
-#if 0
-gboolean
-gpa_list_test_item_by_id (GPAList *list, const gchar *id)
-{
-	GPAItem *item;
-
-	g_return_val_if_fail (list != NULL, FALSE);
-	g_return_val_if_fail (GPA_IS_LIST (list), FALSE);
-	g_return_val_if_fail (id != NULL, FALSE);
-	g_return_val_if_fail (*id != '\0', FALSE);
-
-	for (item = list->items; item != NULL; item = item->next) {
-		if (item->id && !strcmp (item->id, id)) return TRUE;
-	}
-
-	return FALSE;
-}
-
-GPANode *
-gpa_list_get_item_by_id (GPAList *list, const gchar *id)
-{
-	GPAItem *item;
-
-	g_return_val_if_fail (list != NULL, NULL);
-	g_return_val_if_fail (GPA_IS_LIST (list), NULL);
-	g_return_val_if_fail (id != NULL, NULL);
-	g_return_val_if_fail (*id != '\0', NULL);
-
-	for (item = list->items; item != NULL; item = item->next) {
-		if (item->id && !strcmp (item->id, id)) {
-			gpa_node_ref (GPA_NODE (item));
-			return GPA_NODE (item);
-		}
-	}
-
-	return NULL;
-}
-#endif
-
