@@ -194,11 +194,7 @@ gpa_root_get_child (GPANode *node, GPANode *ref)
 	root = GPA_ROOT (node);
 
 	g_return_val_if_fail (root->printers != NULL, NULL);
-#if 0
-	/* This does not work anymore, as children do not have parent */
-	g_return_val_if_fail (!ref || (ref->parent == node), NULL);
-#endif
-
+	
 	if (!ref) {
 		if (root->vendors) {
 			return gpa_node_ref (root->vendors);
@@ -297,15 +293,4 @@ gpa_root_get (void)
 	return root;
 }
 
-#if 0
-GPARoot *
-gpa_root (void)
-{
-	if (!root_instance) {
-		gpa_node_new (GPA_TYPE_ROOT, "Globals");
-	}
-
-	return (GPARoot *) root_instance;
-}
-#endif
 
