@@ -154,7 +154,7 @@ gpa_vendor_get_child (GPANode *node, GPANode *ref)
 {
 	GPAVendor *vendor;
 	GPANode *child;
-
+	
 	vendor = GPA_VENDOR (node);
 
 	child = NULL;
@@ -166,7 +166,8 @@ gpa_vendor_get_child (GPANode *node, GPANode *ref)
 		child = (GPANode *) vendor->models;
 	}
 
-	if (child) gpa_node_ref (child);
+	if (child)
+		gpa_node_ref (child);
 
 	return child;
 }
